@@ -780,16 +780,20 @@ RTDECL(int) RTFileCreateTempSecure(char *pszTemplate);
 /**
  * Opens a new file with a unique name in the temp directory.
  *
+ * Unlike the other temp file creation APIs, this does not allow you any control
+ * over the name.  Nor do you have to figure out where the temporary directory
+ * is.
+ *
  * @returns iprt status code.
- * @param   pszTemplate     The file name template on input. The actual file
- *                          name on success. Empty string on failure.
- * @param   fMode           The mode to create the file with.  Use 0600 unless
- *                          you have reason not to.
+ * @param   phFile          Where to return the handle to the file.
+ * @param   pszFilename     Where to return the name (+path) of the file .
+ * @param   cbFilename      The size of the buffer @a pszFilename points to.
+ * @param   fOpen           The RTFILE_O_XXX flags to open the file with.
  *
  * @remarks If actual control over the filename or location is required, we'll
  *          create an extended edition of this API.
  */
-RTDECL(int) RTFileOpenTemp(PRTFILE pFile, char *pszFilename, size_t cbFilename, uint64_t fOpen);
+RTDECL(int) RTFileOpenTemp(PRTFILE phFile, char *pszFilename, size_t cbFilename, uint64_t fOpen);
 
 
 /** @page   pg_rt_filelock      RT File locking API description
@@ -1004,7 +1008,7 @@ RTDECL(int) RTFileGetOwner(RTFILE File, uint32_t *pUid, uint32_t *pGid);
  *
  * @returns iprt status code.
  * @param   File        Handle to the file.
- * @param   iRequest    IOCTL request to carry out.
+ * @param   ulRequest   IOCTL request to carry out.
  * @param   pvData      IOCTL data.
  * @param   cbData      Size of the IOCTL data.
  * @param   piRet       Return value of the IOCTL request.
