@@ -1,5 +1,5 @@
-/** @file $Id: vboxvideo_drm.c $
- *
+/* $Id: vboxvideo_drm.c $ */
+/** @file
  * VirtualBox Additions Linux kernel driver, DRM support
  */
 
@@ -49,43 +49,26 @@
  *    Gareth Hughes <gareth@valinux.com>
  */
 
-#include <linux/version.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
-# include <generated/autoconf.h>
-#else
-# ifndef AUTOCONF_INCLUDED
-#  include <linux/autoconf.h>
-# endif
-#endif
-#include <linux/module.h>
 #include "version-generated.h"
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
-
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 32)
-#  ifdef RHEL_RELEASE_CODE
-#   if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 1)
-#    define DRM_RHEL61
-#   endif
-#   if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 3)
-#    define DRM_RHEL63
-#   endif
-#   if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(6, 4)
-#    define DRM_RHEL64
-#   endif
-#  endif
-# endif
-
-#include "drm/drmP.h"
+#include <linux/module.h>
+#include <linux/version.h>
+#include <drm/drmP.h>
 #include "vboxvideo_drm.h"
 
-# ifndef RHEL_RELEASE_CODE
-#  if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 39) && LINUX_VERSION_CODE < KERNEL_VERSION(3, 3, 0)
-#   if defined(DRM_MODE_OBJECT_PLANE) && defined(DRM_IOCTL_MODE_ADDFB2)
-#    define DRM_DEBIAN_34ON32
-#   endif
-#  endif
-# endif
+/* This definition and the file-operations-as-pointer change were both added in
+ * kernel 3.3.  All back-ports of the structure change to distribution kernels
+ * that I have checked also back-ported the definition at the same time. */
+#ifdef DRM_IOCTL_MODE_ADDFB2
+# define DRM_FOPS_AS_POINTER
+#endif
+
+/* The first of these was introduced when drm was generalised to work with
+ * non-PCI buses, but was removed between 3.15 and 3.16.  The second is a
+ * random definition introduced in the mean-time. */
+#if defined(DRIVER_BUS_PCI) || defined(DRIVER_PRIME)
+# define DRM_NEW_BUS_INIT 1
+#endif
 
 static struct pci_device_id pciidlist[] = {
         vboxvideo_PCI_IDS
@@ -95,13 +78,10 @@ MODULE_DEVICE_TABLE(pci, pciidlist);
 
 int vboxvideo_driver_load(struct drm_device * dev, unsigned long flags)
 {
-# if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 28)
-    return drm_vblank_init(dev, 1);
-#else
     return 0;
-#endif
 }
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0) || defined(DRM_RHEL63) || defined(DRM_DEBIAN_34ON32)
+
+#ifdef DRM_FOPS_AS_POINTER
 /* since linux-3.3.0-rc1 drm_driver::fops is pointer */
 static struct file_operations driver_fops =
 {
@@ -109,9 +89,13 @@ static struct file_operations driver_fops =
         .open = drm_open,
         .release = drm_release,
         .unlocked_ioctl = drm_ioctl,
+# if LINUX_VERSION_CODE < KERNEL_VERSION(3, 18, 0)
+        /* This shouldn't be necessary even for old kernels as there is
+         * nothing sensible to mmap. But we play safe and keep it for
+         * legacy reasons. */
         .mmap = drm_mmap,
+# endif
         .poll = drm_poll,
-        .fasync = drm_fasync,
 };
 #endif
 
@@ -119,15 +103,13 @@ static struct drm_driver driver =
 {
     /* .driver_features = DRIVER_USE_MTRR, */
     .load = vboxvideo_driver_load,
-#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 6, 0) && !defined(DRM_RHEL64)
-    .reclaim_buffers = drm_core_reclaim_buffers,
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 18, 0)
+    /* If this is missing a warning gets printed to dmesg.  We will not
+     * attempt to make kernels work to which the change (915b4d11b) got back-
+     * ported, as the problem is only cosmetic. */
+    .set_busid = drm_pci_set_busid,
 #endif
-    /* As of Linux 2.6.37, always the internal functions are used. */
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 37) && !defined(DRM_RHEL61)
-    .get_map_ofs = drm_core_get_map_ofs,
-    .get_reg_ofs = drm_core_get_reg_ofs,
-#endif
-# if LINUX_VERSION_CODE < KERNEL_VERSION(3, 3, 0) && !defined(DRM_RHEL63) && !defined(DRM_DEBIAN_34ON32)
+# ifndef DRM_FOPS_AS_POINTER
     .fops =
     {
         .owner = THIS_MODULE,
@@ -135,19 +117,18 @@ static struct drm_driver driver =
         .release = drm_release,
         /* This was changed with Linux 2.6.33 but Fedora backported this
          * change to their 2.6.32 kernel. */
-#if defined(DRM_UNLOCKED) || LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33)
+#if defined(DRM_UNLOCKED)
         .unlocked_ioctl = drm_ioctl,
 #else
         .ioctl = drm_ioctl,
 #endif
         .mmap = drm_mmap,
         .poll = drm_poll,
-        .fasync = drm_fasync,
     },
-#else /* LINUX_VERSION_CODE >= KERNEL_VERSION(3, 3, 0) || defined(DRM_RHEL63) || defined(DRM_DEBIAN_34ON32) */
+#else /* defined(DRM_FOPS_AS_POINTER) */
     .fops = &driver_fops,
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39) && !defined(DRM_RHEL61)
+#ifndef DRM_NEW_BUS_INIT
     .pci_driver =
     {
         .name = DRIVER_NAME,
@@ -162,7 +143,7 @@ static struct drm_driver driver =
     .patchlevel = DRIVER_PATCHLEVEL,
 };
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 39) || defined(DRM_RHEL61)
+#ifdef DRM_NEW_BUS_INIT
 static struct pci_driver pci_driver =
 {
     .name = DRIVER_NAME,
@@ -172,7 +153,7 @@ static struct pci_driver pci_driver =
 
 static int __init vboxvideo_init(void)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39) && !defined(DRM_RHEL61)
+#ifndef DRM_NEW_BUS_INIT
     return drm_init(&driver);
 #else
     return drm_pci_init(&driver, &pci_driver);
@@ -181,7 +162,7 @@ static int __init vboxvideo_init(void)
 
 static void __exit vboxvideo_exit(void)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 39) && !defined(DRM_RHEL61)
+#ifndef DRM_NEW_BUS_INIT
     drm_exit(&driver);
 #else
     drm_pci_exit(&driver, &pci_driver);
@@ -193,9 +174,6 @@ module_exit(vboxvideo_exit);
 
 MODULE_AUTHOR(DRIVER_AUTHOR);
 MODULE_DESCRIPTION(DRIVER_DESC);
-
-#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27) */
-
 #ifdef MODULE_VERSION
 MODULE_VERSION(VBOX_VERSION_STRING);
 #endif
