@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# Linux Additions kernel module init script ($Revision: 102425 $)
+# Linux Additions kernel module init script ($Revision: 104011 $)
 #
 
 #
@@ -63,8 +63,6 @@ elif [ -f /etc/SuSE-release ]; then
     system=suse
 elif [ -f /etc/gentoo-release ]; then
     system=gentoo
-elif [ -f /etc/lfs-release -a -d /etc/rc.d/init.d ]; then
-    system=lfs
 else
     system=other
 fi
@@ -119,19 +117,6 @@ if [ "$system" = "gentoo" ]; then
     fi
 fi
 
-if [ "$system" = "lfs" ]; then
-    . /etc/rc.d/init.d/functions
-    fail_msg() {
-        echo_failure
-    }
-    succ_msg() {
-        echo_ok
-    }
-    begin() {
-        echo $1
-    }
-fi
-
 if [ "$system" = "other" ]; then
     fail_msg() {
         echo " ...fail!"
