@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -63,11 +63,11 @@ static int sf_reg_read_aux(const char *caller, struct sf_glob_info *sf_g,
     /** @todo bird: yes, kmap() and kmalloc() input only. Since the buffer is
      *        contiguous in physical memory (kmalloc or single page), we should
      *        use a physical address here to speed things up. */
-    int rc = vboxCallRead(&client_handle, &sf_g->map, sf_r->handle,
+    int rc = VbglR0SfRead(&client_handle, &sf_g->map, sf_r->handle,
                           pos, nread, buf, false /* already locked? */);
     if (RT_FAILURE(rc))
     {
-        LogFunc(("vboxCallRead failed. caller=%s, rc=%Rrc\n", caller, rc));
+        LogFunc(("VbglR0SfRead failed. caller=%s, rc=%Rrc\n", caller, rc));
         return -EPROTO;
     }
     return 0;
@@ -80,11 +80,11 @@ static int sf_reg_write_aux(const char *caller, struct sf_glob_info *sf_g,
     /** @todo bird: yes, kmap() and kmalloc() input only. Since the buffer is
      *        contiguous in physical memory (kmalloc or single page), we should
      *        use a physical address here to speed things up. */
-    int rc = vboxCallWrite(&client_handle, &sf_g->map, sf_r->handle,
+    int rc = VbglR0SfWrite(&client_handle, &sf_g->map, sf_r->handle,
                            pos, nwritten, buf, false /* already locked? */);
     if (RT_FAILURE(rc))
     {
-        LogFunc(("vboxCallWrite failed. caller=%s, rc=%Rrc\n",
+        LogFunc(("VbglR0SfWrite failed. caller=%s, rc=%Rrc\n",
                     caller, rc));
         return -EPROTO;
     }
@@ -108,7 +108,7 @@ static ssize_t sf_reg_read(struct file *file, char *buf, size_t size, loff_t *of
     size_t tmp_size;
     size_t left = size;
     ssize_t total_bytes_read = 0;
-    struct inode *inode = file->f_dentry->d_inode;
+    struct inode *inode = GET_F_DENTRY(file)->d_inode;
     struct sf_glob_info *sf_g = GET_GLOB_INFO(inode->i_sb);
     struct sf_reg_info *sf_r = file->private_data;
     loff_t pos = *off;
@@ -183,7 +183,7 @@ static ssize_t sf_reg_write(struct file *file, const char *buf, size_t size, lof
     size_t tmp_size;
     size_t left = size;
     ssize_t total_bytes_written = 0;
-    struct inode *inode = file->f_dentry->d_inode;
+    struct inode *inode = GET_F_DENTRY(file)->d_inode;
     struct sf_inode_info *sf_i = GET_INODE_INFO(inode);
     struct sf_glob_info *sf_g = GET_GLOB_INFO(inode->i_sb);
     struct sf_reg_info *sf_r = file->private_data;
@@ -373,12 +373,12 @@ static int sf_reg_open(struct inode *inode, struct file *file)
     }
 
     params.Info.Attr.fMode = inode->i_mode;
-    LogFunc(("sf_reg_open: calling vboxCallCreate, file %s, flags=%#x, %#x\n",
+    LogFunc(("sf_reg_open: calling VbglR0SfCreate, file %s, flags=%#x, %#x\n",
               sf_i->path->String.utf8 , file->f_flags, params.CreateFlags));
-    rc = vboxCallCreate(&client_handle, &sf_g->map, sf_i->path, &params);
+    rc = VbglR0SfCreate(&client_handle, &sf_g->map, sf_i->path, &params);
     if (RT_FAILURE(rc))
     {
-        LogFunc(("vboxCallCreate failed flags=%d,%#x rc=%Rrc\n",
+        LogFunc(("VbglR0SfCreate failed flags=%d,%#x rc=%Rrc\n",
                   file->f_flags, params.CreateFlags, rc));
         kfree(sf_r);
         return -RTErrConvertToErrno(rc);
@@ -438,9 +438,9 @@ static int sf_reg_release(struct inode *inode, struct file *file)
         && filemap_fdatawrite(inode->i_mapping) != -EIO)
         filemap_fdatawait(inode->i_mapping);
 #endif
-    rc = vboxCallClose(&client_handle, &sf_g->map, sf_r->handle);
+    rc = VbglR0SfClose(&client_handle, &sf_g->map, sf_r->handle);
     if (RT_FAILURE(rc))
-        LogFunc(("vboxCallClose failed rc=%Rrc\n", rc));
+        LogFunc(("VbglR0SfClose failed rc=%Rrc\n", rc));
 
     kfree(sf_r);
     sf_i->file = NULL;
@@ -454,7 +454,7 @@ static int sf_reg_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
 #elif LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 static struct page *sf_reg_nopage(struct vm_area_struct *vma, unsigned long vaddr, int *type)
 # define SET_TYPE(t) *type = (t)
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION (2, 6, 0) */
+#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0) */
 static struct page *sf_reg_nopage(struct vm_area_struct *vma, unsigned long vaddr, int unused)
 # define SET_TYPE(t)
 #endif
@@ -465,7 +465,7 @@ static struct page *sf_reg_nopage(struct vm_area_struct *vma, unsigned long vadd
     uint32_t nread = PAGE_SIZE;
     int err;
     struct file *file = vma->vm_file;
-    struct inode *inode = file->f_dentry->d_inode;
+    struct inode *inode = GET_F_DENTRY(file)->d_inode;
     struct sf_glob_info *sf_g = GET_GLOB_INFO(inode->i_sb);
     struct sf_reg_info *sf_r = file->private_data;
 
@@ -481,7 +481,7 @@ static struct page *sf_reg_nopage(struct vm_area_struct *vma, unsigned long vadd
     }
 #endif
 
-    /* Don't use GFP_HIGHUSER as long as sf_reg_read_aux() calls vboxCallRead()
+    /* Don't use GFP_HIGHUSER as long as sf_reg_read_aux() calls VbglR0SfRead()
      * which works on virtual addresses. On Linux cannot reliably determine the
      * physical address for high memory, see rtR0MemObjNativeLockKernel(). */
     page = alloc_page(GFP_USER);
@@ -574,8 +574,13 @@ struct file_operations sf_reg_fops =
 # else
     .sendfile    = generic_file_sendfile,
 # endif
+# if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 16, 0)
+    .read_iter   = generic_file_read_iter,
+    .write_iter  = generic_file_write_iter,
+# else
     .aio_read    = generic_file_aio_read,
     .aio_write   = generic_file_aio_write,
+# endif
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 35)
     .fsync       = noop_fsync,
 # else
@@ -600,7 +605,7 @@ struct inode_operations sf_reg_iops =
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 static int sf_readpage(struct file *file, struct page *page)
 {
-    struct inode *inode = file->f_dentry->d_inode;
+    struct inode *inode = GET_F_DENTRY(file)->d_inode;
     struct sf_glob_info *sf_g = GET_GLOB_INFO(inode->i_sb);
     struct sf_reg_info *sf_r = file->private_data;
     uint32_t nread = PAGE_SIZE;
