@@ -1,5 +1,5 @@
 /*
- * $Id$
+ * $Id: damagestr.h $
  *
  * Copyright © 2003 Keith Packard
  *
@@ -30,7 +30,6 @@
 #define _DAMAGESTR_H_
 
 #include "damage.h"
-#include "gcstruct.h"
 #ifdef RENDER
 # include "picturestr.h"
 #endif
@@ -48,9 +47,6 @@ typedef struct _damage {
     
     DamageReportFunc	damageReport;
     DamageDestroyFunc	damageDestroy;
-
-    Bool		reportAfter;
-    RegionRec		pendingDamage;
 } DamageRec;
 
 typedef struct _damageScrPriv {
