@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2012 Oracle Corporation
+ * Copyright (C) 2011-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -44,6 +44,30 @@ typedef struct
     uint32_t    len;
 } ahci_prdt;
 
+/**
+ * SATA D2H FIS (Device to Host Frame Information Structure).
+ */
+typedef struct {
+    uint8_t     fis_type;   /* 34h */
+    uint8_t     intr;       /* Bit 6 indicates interrupt status. */
+    uint8_t     status;     /* Status register. */
+    uint8_t     error;      /* Error register. */
+    uint8_t     sec_no;     /* Sector number register. */
+    uint8_t     cyl_lo;     /* Cylinder low register. */
+    uint8_t     cyl_hi;     /* Cylinder high register. */
+    uint8_t     dev_hd;     /* Device/head register. */
+    uint8_t     sec_no_exp; /* Expanded sector number register. */
+    uint8_t     cyl_lo_exp; /* Expanded cylinder low register. */
+    uint8_t     cyl_hi_exp; /* Expanded cylinder high register. */
+    uint8_t     resvd0;
+    uint8_t     sec_cn;     /* Sector count register. */
+    uint8_t     sec_cn_exp; /* Expanded sector count register. */
+    uint16_t    resvd1;
+    uint32_t    resvd2;
+} fis_d2h;
+
+ct_assert(sizeof(fis_d2h) == 20);
+
 /**
  * AHCI controller data.
  */
@@ -304,8 +328,7 @@ static void ahci_port_cmd_sync(ahci_t __far *ahci, uint8_t val)
         /* Disable command engine. */
         ahci_ctrl_clear_bits(io_base, AHCI_PORT_REG(port, AHCI_REG_PORT_CMD),
                              AHCI_REG_PORT_CMD_ST);
-
-        /** @todo: Examine status. */
+        /* Caller must examine status. */
     }
     else
         DBG_AHCI("AHCI: Invalid port given\n");
@@ -314,12 +337,13 @@ static void ahci_port_cmd_sync(ahci_t __far *ahci, uint8_t val)
 /**
  * Issue command to device.
  */
-static void ahci_cmd_data(bio_dsk_t __far *bios_dsk, uint8_t cmd)
+static uint16_t ahci_cmd_data(bio_dsk_t __far *bios_dsk, uint8_t cmd)
 {
     ahci_t __far    *ahci  = bios_dsk->ahci_seg :> 0;
     uint16_t        n_sect = bios_dsk->drqp.nsect;
     uint16_t        sectsz = bios_dsk->drqp.sect_sz;
     uint16_t        prdt_idx;
+    fis_d2h __far   *d2h;
 
     _fmemset(&ahci->abCmd[0], 0, sizeof(ahci->abCmd));
 
@@ -344,7 +368,7 @@ static void ahci_cmd_data(bio_dsk_t __far *bios_dsk, uint8_t cmd)
 
     /* Lock memory needed for DMA. */
     ahci->edds.num_avail = NUM_EDDS_SG;
-    DBG_AHCI("AHCI: S/G list for %lu bytes (skip %u)\n", 
+    DBG_AHCI("AHCI: S/G list for %lu bytes (skip %u)\n",
              (uint32_t)n_sect * sectsz, bios_dsk->drqp.skip_a);
     vds_build_sg_list(&ahci->edds, bios_dsk->drqp.buffer, (uint32_t)n_sect * sectsz);
 
@@ -383,8 +407,13 @@ static void ahci_cmd_data(bio_dsk_t __far *bios_dsk, uint8_t cmd)
 
     ahci_port_cmd_sync(ahci, cmd);
 
+    /* Examine operation status. */
+    d2h = (void __far *)&ahci->abFisRecv[0x40];
+    DBG_AHCI("AHCI: ERR=%02x, STAT=%02x, SCNT=%02x\n", d2h->error, d2h->status, d2h->sec_cn);
+
     /* Unlock the buffer again. */
     vds_free_sg_list(&ahci->edds);
+    return d2h->error ? 4 : 0;
 }
 
 /**
@@ -483,12 +512,13 @@ static void ahci_port_init(ahci_t __far *ahci, uint8_t u8Port)
  * Read sectors from an attached AHCI device.
  *
  * @returns status code.
- * @param   bios_dsk    Pointer to disk request packet (in the 
+ * @param   bios_dsk    Pointer to disk request packet (in the
  *                      EBDA).
  */
 int ahci_read_sectors(bio_dsk_t __far *bios_dsk)
 {
     uint16_t        device_id;
+    uint16_t        rc;
 
     device_id = VBOX_GET_AHCI_DEVICE(bios_dsk->drqp.dev_id);
     if (device_id > BX_MAX_AHCI_DEVICES)
@@ -500,26 +530,27 @@ int ahci_read_sectors(bio_dsk_t __far *bios_dsk)
 
     high_bits_save(bios_dsk->ahci_seg :> 0);
     ahci_port_init(bios_dsk->ahci_seg :> 0, bios_dsk->ahcidev[device_id].port);
-    ahci_cmd_data(bios_dsk, AHCI_CMD_READ_DMA_EXT);
+    rc = ahci_cmd_data(bios_dsk, AHCI_CMD_READ_DMA_EXT);
     DBG_AHCI("%s: transferred %lu bytes\n", __func__, ((ahci_t __far *)(bios_dsk->ahci_seg :> 0))->aCmdHdr[1]);
     bios_dsk->drqp.trsfsectors = bios_dsk->drqp.nsect;
 #ifdef DMA_WORKAROUND
     rep_movsw(bios_dsk->drqp.buffer, bios_dsk->drqp.buffer, bios_dsk->drqp.nsect * 512 / 2);
 #endif
     high_bits_restore(bios_dsk->ahci_seg :> 0);
-    return 0;   //@todo!!
+    return rc;
 }
 
 /**
  * Write sectors to an attached AHCI device.
  *
  * @returns status code.
- * @param   bios_dsk    Pointer to disk request packet (in the 
+ * @param   bios_dsk    Pointer to disk request packet (in the
  *                      EBDA).
  */
 int ahci_write_sectors(bio_dsk_t __far *bios_dsk)
 {
     uint16_t        device_id;
+    uint16_t        rc;
 
     device_id = VBOX_GET_AHCI_DEVICE(bios_dsk->drqp.dev_id);
     if (device_id > BX_MAX_AHCI_DEVICES)
@@ -531,11 +562,11 @@ int ahci_write_sectors(bio_dsk_t __far *bios_dsk)
 
     high_bits_save(bios_dsk->ahci_seg :> 0);
     ahci_port_init(bios_dsk->ahci_seg :> 0, bios_dsk->ahcidev[device_id].port);
-    ahci_cmd_data(bios_dsk, AHCI_CMD_WRITE_DMA_EXT);
+    rc = ahci_cmd_data(bios_dsk, AHCI_CMD_WRITE_DMA_EXT);
     DBG_AHCI("%s: transferred %lu bytes\n", __func__, ((ahci_t __far *)(bios_dsk->ahci_seg :> 0))->aCmdHdr[1]);
     bios_dsk->drqp.trsfsectors = bios_dsk->drqp.nsect;
     high_bits_restore(bios_dsk->ahci_seg :> 0);
-    return 0;   //@todo!!
+    return rc;
 }
 
 //@todo: move
@@ -543,7 +574,7 @@ int ahci_write_sectors(bio_dsk_t __far *bios_dsk)
 #define ATA_DATA_IN      0x01
 #define ATA_DATA_OUT     0x02
 
-uint16_t ahci_cmd_packet(uint16_t device_id, uint8_t cmdlen, char __far *cmdbuf, 
+uint16_t ahci_cmd_packet(uint16_t device_id, uint8_t cmdlen, char __far *cmdbuf,
                          uint16_t skip_b, uint32_t length, uint8_t inout, char __far *buffer)
 {
     bio_dsk_t __far *bios_dsk = read_word(0x0040, 0x000E) :> &EbdaData->bdisk;
@@ -565,7 +596,7 @@ uint16_t ahci_cmd_packet(uint16_t device_id, uint8_t cmdlen, char __far *cmdbuf,
     device_id = VBOX_GET_AHCI_DEVICE(device_id);
 
     DBG_AHCI("%s: reading %lu bytes, skip %u/%u, device %d, port %d\n", __func__,
-             length, bios_dsk->drqp.skip_b, bios_dsk->drqp.skip_a, 
+             length, bios_dsk->drqp.skip_b, bios_dsk->drqp.skip_a,
              device_id, bios_dsk->ahcidev[device_id].port);
     DBG_AHCI("%s: reading %u %u-byte sectors\n", __func__,
              bios_dsk->drqp.nsect, bios_dsk->drqp.sect_sz);
@@ -604,10 +635,9 @@ uint16_t ahci_cmd_packet(uint16_t device_id, uint8_t cmdlen, char __far *cmdbuf,
     high_bits_restore(ahci);
 
     return ahci->aCmdHdr[1] == 0 ? 4 : 0;
-//    return 0;   //@todo!!
 }
 
-static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
+void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
 {
     uint32_t            val;
     bio_dsk_t __far     *bios_dsk;
@@ -626,6 +656,14 @@ static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
 
     /* Check if there is a device on the port. */
     VBOXAHCI_PORT_READ_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SSTS, val);
+    if (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0)
+        return; /* No device detected. */
+
+    do
+    {
+        VBOXAHCI_PORT_READ_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SSTS, val);
+    } while (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0x1);
+
     if (ahci_ctrl_extract_bits(val, 0xfL, 0) == 0x3)
     {
         uint8_t     abBuffer[0x0200];
@@ -633,6 +671,9 @@ static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
         uint8_t     cdcount;
         uint8_t     removable;
 
+        /* Clear all errors after the reset. */
+        VBOXAHCI_PORT_WRITE_REG(ahci->iobase, u8Port, AHCI_REG_PORT_SERR, 0xffffffff);
+
         devcount_ahci = bios_dsk->ahci_devcnt;
 
         DBG_AHCI("AHCI: Device detected on port %d\n", u8Port);
@@ -650,7 +691,7 @@ static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
             {
                 uint32_t    sectors;
                 uint16_t    cylinders, heads, spt;
-                uint16_t    lcylinders, lheads, lspt;
+                chs_t       lgeo;
                 uint8_t     idxCmosChsBase;
 
                 DBG_AHCI("AHCI: Detected hard disk\n");
@@ -710,23 +751,17 @@ static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
                 }
                 if (idxCmosChsBase && inb_cmos(idxCmosChsBase+7))
                 {
-                    lcylinders = inb_cmos(idxCmosChsBase + 0) + (inb_cmos(idxCmosChsBase + 1) << 8);
-                    lheads     = inb_cmos(idxCmosChsBase + 2);
-                    lspt       = inb_cmos(idxCmosChsBase + 7);
+                    lgeo.cylinders = inb_cmos(idxCmosChsBase + 0) + (inb_cmos(idxCmosChsBase + 1) << 8);
+                    lgeo.heads     = inb_cmos(idxCmosChsBase + 2);
+                    lgeo.spt       = inb_cmos(idxCmosChsBase + 7);
                 }
                 else
-                {
-                    //@todo: What should we really do if logical geometry isn't in the CMOS?
-                    lcylinders = cylinders;
-                    lheads     = heads;
-                    lspt       = spt;
-                }
-                BX_INFO("AHCI %d-P#%d: PCHS=%u/%d/%d LCHS=%u/%u/%u %ld sectors\n", devcount_ahci,
-                        u8Port, cylinders, heads, spt, lcylinders, lheads, lspt, sectors);
+                    set_geom_lba(&lgeo, sectors);   /* Default EDD-style translated LBA geometry. */
 
-                bios_dsk->devices[hd_index].lchs.heads     = lheads;
-                bios_dsk->devices[hd_index].lchs.cylinders = lcylinders;
-                bios_dsk->devices[hd_index].lchs.spt       = lspt;
+                BX_INFO("AHCI %d-P#%d: PCHS=%u/%u/%u LCHS=%u/%u/%u %lu sectors\n", devcount_ahci,
+                        u8Port, cylinders, heads, spt, lgeo.cylinders, lgeo.heads, lgeo.spt, sectors);
+
+                bios_dsk->devices[hd_index].lchs = lgeo;
 
                 /* Store the ID of the disk in the BIOS hdidmap. */
                 hdcount = bios_dsk->hdcount;
@@ -737,7 +772,7 @@ static void ahci_port_detect_device(ahci_t __far *ahci, uint8_t u8Port)
                 /* Update hdcount in the BDA. */
                 hdcount = read_byte(0x40, 0x75);
                 hdcount++;
-                write_byte(0x40, 0x75, hdcount);                
+                write_byte(0x40, 0x75, hdcount);
             }
             else if (val == 0xeb140101)
             {
@@ -812,7 +847,7 @@ static int ahci_hba_init(uint16_t io_base)
     uint16_t            ahci_seg;
     bio_dsk_t __far     *bios_dsk;
     ahci_t __far        *ahci;
-    
+
 
     ebda_seg = read_word(0x0040, 0x000E);
     bios_dsk = ebda_seg :> &EbdaData->bdisk;
@@ -829,7 +864,7 @@ static int ahci_hba_init(uint16_t io_base)
         DBG_AHCI("AHCI: Could not allocate 1K of memory, can't boot from controller\n");
         return 0;
     }
-    DBG_AHCI("AHCI: ahci_seg=%04x, size=%04x, pointer at EBDA:%04x (EBDA size=%04x)\n", 
+    DBG_AHCI("AHCI: ahci_seg=%04x, size=%04x, pointer at EBDA:%04x (EBDA size=%04x)\n",
              ahci_seg, sizeof(ahci_t), (uint16_t)&EbdaData->bdisk.ahci_seg, sizeof(ebda_data_t));
 
     bios_dsk->ahci_seg    = ahci_seg;
@@ -849,7 +884,7 @@ static int ahci_hba_init(uint16_t io_base)
     do
     {
         AHCI_READ_REG(io_base, AHCI_REG_GHC, val);
-    } while (val & AHCI_GHC_HR != 0);
+    } while ((val & AHCI_GHC_HR) != 0);
 
     AHCI_READ_REG(io_base, AHCI_REG_CAP, val);
     cPorts = ahci_ctrl_extract_bits(val, 0x1f, 0) + 1; /* Extract number of ports.*/
