@@ -1,8 +1,6 @@
 /* $Id: UIDownloaderUserManual.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIDownloaderUserManual class implementation
+ * VBox Qt GUI - UIDownloaderUserManual class implementation.
  */
 
 /*
@@ -17,17 +15,24 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* Global includes: */
-#include <QDir>
-#include <QFile>
+# include <QDir>
+# include <QFile>
 
 /* Local includes: */
-#include "UIDownloaderUserManual.h"
-#include "UINetworkReply.h"
-#include "QIFileDialog.h"
-#include "VBoxGlobal.h"
-#include "UIMessageCenter.h"
-#include "UIModalWindowManager.h"
+# include "UIDownloaderUserManual.h"
+# include "UINetworkReply.h"
+# include "QIFileDialog.h"
+# include "VBoxGlobal.h"
+# include "UIMessageCenter.h"
+# include "UIModalWindowManager.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 
 /* static */
 UIDownloaderUserManual* UIDownloaderUserManual::m_spInstance = 0;
@@ -52,9 +57,6 @@ UIDownloaderUserManual::UIDownloaderUserManual()
     if (!m_spInstance)
         m_spInstance = this;
 
-    /* Set description: */
-    setDescription(tr("VirtualBox User Manual"));
-
     /* Compose User Manual filename: */
     QString strUserManualFullFileName = vboxGlobal().helpFile();
     QString strUserManualShortFileName = QFileInfo(strUserManualFullFileName).fileName();
@@ -64,7 +66,7 @@ UIDownloaderUserManual::UIDownloaderUserManual()
     addSource(QString("http://download.virtualbox.org/virtualbox/") + strUserManualShortFileName);
 
     /* Set target: */
-    QString strUserManualDestination = QDir(vboxGlobal().virtualBox().GetHomeFolder()).absoluteFilePath(strUserManualShortFileName);
+    QString strUserManualDestination = QDir(vboxGlobal().homeFolder()).absoluteFilePath(strUserManualShortFileName);
     setTarget(strUserManualDestination);
 }
 
@@ -75,9 +77,15 @@ UIDownloaderUserManual::~UIDownloaderUserManual()
         m_spInstance = 0;
 }
 
+/* virtual override */
+const QString UIDownloaderUserManual::description() const
+{
+    return UIDownloader::description().arg(tr("VirtualBox User Manual"));
+}
+
 bool UIDownloaderUserManual::askForDownloadingConfirmation(UINetworkReply *pReply)
 {
-    return msgCenter().confirmDownloadUserManual(source().toString(), pReply->header(QNetworkRequest::ContentLengthHeader).toInt());
+    return msgCenter().confirmDownloadUserManual(source().toString(), pReply->header(UINetworkReply::ContentLengthHeader).toInt());
 }
 
 void UIDownloaderUserManual::handleDownloadedObject(UINetworkReply *pReply)
