@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,20 +24,12 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
-#define LOG_GROUP LOG_GROUP_SUP_DRV
-/*
- * Deal with conflicts first.
- * PVM - BSD mess, that FreeBSD has correct a long time ago.
- * iprt/types.h before sys/param.h - prevents UINT32_C and friends.
- */
-#include <iprt/types.h>
-#include <sys/param.h>
-#undef PVM
 
-#include <IOKit/IOLib.h> /* Assert as function */
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
+#define LOG_GROUP LOG_GROUP_SUP_DRV
+#include "../../../Runtime/r0drv/darwin/the-darwin-kernel.h"
 
 #include "../SUPDrvInternal.h"
 #include <VBox/version.h>
@@ -78,10 +70,15 @@ RT_C_DECLS_BEGIN
 RT_C_DECLS_END
 #endif
 
+/* Temporary debugging - very temporary... */
+#define VBOX_PROC_SELFNAME_LEN  (20)
+#define VBOX_RETRIEVE_CUR_PROC_NAME(_name)  char _name[VBOX_PROC_SELFNAME_LEN]; \
+                                            proc_selfname(pszProcName, VBOX_PROC_SELFNAME_LEN)
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 
 /** The system device node name. */
 #define DEVICE_NAME_SYS     "vboxdrv"
@@ -90,9 +87,9 @@ RT_C_DECLS_END
 
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 RT_C_DECLS_BEGIN
 static kern_return_t    VBoxDrvDarwinStart(struct kmod_info *pKModInfo, void *pvData);
 static kern_return_t    VBoxDrvDarwinStop(struct kmod_info *pKModInfo, void *pvData);
@@ -112,9 +109,9 @@ static int              vboxdrvDarwinResolveSymbols(void);
 static bool             vboxdrvDarwinCpuHasSMAP(void);
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * The service class.
  * This is just a formality really.
@@ -164,9 +161,9 @@ OSDefineMetaClassAndStructors(org_virtualbox_SupDrvClient, IOUserClient);
 
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /**
  * Declare the module stuff.
  */
@@ -233,6 +230,15 @@ static PFNRT            g_pfnVmxResume = NULL;
 /** Pointer to vmx_use_count. */
 static int volatile    *g_pVmxUseCount = NULL;
 
+#ifdef SUPDRV_WITH_MSR_PROBER
+/** Pointer to rdmsr_carefully if found. Returns 0 on success. */
+static int             (*g_pfnRdMsrCarefully)(uint32_t uMsr, uint32_t *puLow, uint32_t *puHigh) = NULL;
+/** Pointer to rdmsr64_carefully if found. Returns 0 on success. */
+static int             (*g_pfnRdMsr64Carefully)(uint32_t uMsr, uint64_t *uValue) = NULL;
+/** Pointer to wrmsr[64]_carefully if found. Returns 0 on success. */
+static int             (*g_pfnWrMsr64Carefully)(uint32_t uMsr, uint64_t uValue) = NULL;
+#endif
+
 
 /**
  * Start the kernel module.
@@ -348,7 +354,8 @@ static int vboxdrvDarwinResolveSymbols(void)
     if (RT_SUCCESS(rc))
     {
         /*
-         * The VMX stuff - required (for raw-mode).
+         * The VMX stuff - required with raw-mode (in theory for 64-bit on
+         * 32-bit too, but we never did that on darwin).
          */
         int rc1 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "vmx_resume", (void **)&g_pfnVmxResume);
         int rc2 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "vmx_suspend", (void **)&g_pfnVmxSuspend);
@@ -364,7 +371,33 @@ static int vboxdrvDarwinResolveSymbols(void)
             g_pfnVmxResume  = NULL;
             g_pfnVmxSuspend = NULL;
             g_pVmxUseCount  = NULL;
+#ifdef VBOX_WITH_RAW_MODE
             rc = VERR_SYMBOL_NOT_FOUND;
+#endif
+        }
+
+        if (RT_SUCCESS(rc))
+        {
+#ifdef SUPDRV_WITH_MSR_PROBER
+            /*
+             * MSR prober stuff - optional!
+             */
+            int rc2 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "rdmsr_carefully", (void **)&g_pfnRdMsrCarefully);
+            if (RT_FAILURE(rc2))
+                g_pfnRdMsrCarefully = NULL;
+            rc2 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "rdmsr64_carefully", (void **)&g_pfnRdMsr64Carefully);
+            if (RT_FAILURE(rc2))
+                g_pfnRdMsr64Carefully = NULL;
+# ifdef RT_ARCH_AMD64 /* Missing 64 in name, so if implemented on 32-bit it could have different signature. */
+            rc2 = RTR0DbgKrnlInfoQuerySymbol(hKrnlInfo, NULL, "wrmsr_carefully", (void **)&g_pfnWrMsr64Carefully);
+            if (RT_FAILURE(rc2))
+# endif
+                g_pfnWrMsr64Carefully = NULL;
+
+            LogRel(("VBoxDrv: g_pfnRdMsrCarefully=%p g_pfnRdMsr64Carefully=%p g_pfnWrMsr64Carefully=%p\n",
+                    g_pfnRdMsrCarefully, g_pfnRdMsr64Carefully, g_pfnWrMsr64Carefully));
+
+#endif /* SUPDRV_WITH_MSR_PROBER */
         }
 
         RTR0DbgKrnlInfoRelease(hKrnlInfo);
@@ -448,7 +481,7 @@ static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *p
      * The process issuing the request must be the current process.
      */
     RTPROCESS Process = RTProcSelf();
-    if (Process != proc_pid(pProcess))
+    if ((int)Process != proc_pid(pProcess))
         return EIO;
 
     /*
@@ -490,7 +523,7 @@ static int VBoxDrvDarwinOpen(dev_t Dev, int fFlags, int fDevType, struct proc *p
         else
             rc = VERR_GENERAL_FAILURE;
 
-        RTSpinlockReleaseNoInts(g_Spinlock);
+        RTSpinlockRelease(g_Spinlock);
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= 1050
         kauth_cred_unref(&pCred);
 #else  /* 10.4 */
@@ -544,6 +577,20 @@ static int VBoxDrvDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags,
     const unsigned      iHash = SESSION_HASH(Process);
     PSUPDRVSESSION      pSession;
 
+#ifdef VBOX_WITH_EFLAGS_AC_SET_IN_VBOXDRV
+    /*
+     * Refuse all I/O control calls if we've ever detected EFLAGS.AC being cleared.
+     *
+     * This isn't a problem, as there is absolutely nothing in the kernel context that
+     * depend on user context triggering cleanups.  That would be pretty wild, right?
+     */
+    if (RT_UNLIKELY(g_DevExt.cBadContextCalls > 0))
+    {
+        SUPR0Printf("VBoxDrvDarwinIOCtl: EFLAGS.AC=0 detected %u times, refusing all I/O controls!\n", g_DevExt.cBadContextCalls);
+        return EDEVERR;
+    }
+#endif
+
     /*
      * Find the session.
      */
@@ -556,7 +603,7 @@ static int VBoxDrvDarwinIOCtl(dev_t Dev, u_long iCmd, caddr_t pData, int fFlags,
     if (RT_LIKELY(pSession))
         supdrvSessionRetain(pSession);
 
-    RTSpinlockReleaseNoInts(g_Spinlock);
+    RTSpinlockRelease(g_Spinlock);
     if (RT_UNLIKELY(!pSession))
     {
         OSDBGPRINT(("VBoxDrvDarwinIOCtl: WHAT?!? pSession == NULL! This must be a mistake... pid=%d iCmd=%#lx\n",
@@ -598,10 +645,25 @@ static int VBoxDrvDarwinIOCtlSMAP(dev_t Dev, u_long iCmd, caddr_t pData, int fFl
      * Allow VBox R0 code to touch R3 memory. Setting the AC bit disables the
      * SMAP check.
      */
-    RTCCUINTREG uFlags = ASMGetFlags();
-    ASMSetAC();
+    RTCCUINTREG fSavedEfl = ASMAddFlags(X86_EFL_AC);
+
     int rc = VBoxDrvDarwinIOCtl(Dev, iCmd, pData, fFlags, pProcess);
-    ASMSetFlags(uFlags);
+
+#if defined(VBOX_STRICT) || defined(VBOX_WITH_EFLAGS_AC_SET_IN_VBOXDRV)
+    /*
+     * Before we restore AC and the rest of EFLAGS, check if the IOCtl handler code
+     * accidentially modified it or some other important flag.
+     */
+    if (RT_UNLIKELY(   (ASMGetFlags() & (X86_EFL_AC | X86_EFL_IF | X86_EFL_DF | X86_EFL_IOPL))
+                    != ((fSavedEfl    & (X86_EFL_AC | X86_EFL_IF | X86_EFL_DF | X86_EFL_IOPL)) | X86_EFL_AC) ))
+    {
+        char szTmp[48];
+        RTStrPrintf(szTmp, sizeof(szTmp), "iCmd=%#x: %#x->%#x!", iCmd, (uint32_t)fSavedEfl, (uint32_t)ASMGetFlags());
+        supdrvBadContext(&g_DevExt, "SUPDrv-darwin.cpp",  __LINE__, szTmp);
+    }
+#endif
+
+    ASMSetFlags(fSavedEfl);
     return rc;
 }
 
@@ -654,17 +716,20 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
         /*
          * Get the header and figure out how much we're gonna have to read.
          */
+        IPRT_DARWIN_SAVE_EFL_AC();
         SUPREQHDR Hdr;
         pUser = (user_addr_t)*(void **)pData;
         int rc = copyin(pUser, &Hdr, sizeof(Hdr));
         if (RT_UNLIKELY(rc))
         {
             OSDBGPRINT(("VBoxDrvDarwinIOCtlSlow: copyin(%llx,Hdr,) -> %#x; iCmd=%#lx\n", (unsigned long long)pUser, rc, iCmd));
+            IPRT_DARWIN_RESTORE_EFL_AC();
             return rc;
         }
         if (RT_UNLIKELY((Hdr.fFlags & SUPREQHDR_FLAGS_MAGIC_MASK) != SUPREQHDR_FLAGS_MAGIC))
         {
             OSDBGPRINT(("VBoxDrvDarwinIOCtlSlow: bad magic fFlags=%#x; iCmd=%#lx\n", Hdr.fFlags, iCmd));
+            IPRT_DARWIN_SAVE_EFL_AC();
             return EINVAL;
         }
         cbReq = RT_MAX(Hdr.cbIn, Hdr.cbOut);
@@ -673,6 +738,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
                         ||  cbReq > _1M*16))
         {
             OSDBGPRINT(("VBoxDrvDarwinIOCtlSlow: max(%#x,%#x); iCmd=%#lx\n", Hdr.cbIn, Hdr.cbOut, iCmd));
+            IPRT_DARWIN_SAVE_EFL_AC();
             return EINVAL;
         }
 
@@ -685,6 +751,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
         if (RT_UNLIKELY(!pHdr))
         {
             OSDBGPRINT(("VBoxDrvDarwinIOCtlSlow: failed to allocate buffer of %d bytes; iCmd=%#lx\n", cbReq, iCmd));
+            IPRT_DARWIN_RESTORE_EFL_AC();
             return ENOMEM;
         }
         rc = copyin(pUser, pHdr, Hdr.cbIn);
@@ -696,10 +763,12 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
                 IOFreeAligned(pvPageBuf, RT_ALIGN_Z(cbReq, PAGE_SIZE));
             else
                 RTMemTmpFree(pHdr);
+            IPRT_DARWIN_RESTORE_EFL_AC();
             return rc;
         }
         if (Hdr.cbIn < cbReq)
             RT_BZERO((uint8_t *)pHdr + Hdr.cbIn, cbReq - Hdr.cbIn);
+        IPRT_DARWIN_RESTORE_EFL_AC();
     }
     else
     {
@@ -718,6 +787,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
          */
         if (pUser)
         {
+            IPRT_DARWIN_SAVE_EFL_AC();
             uint32_t cbOut = pHdr->cbOut;
             if (cbOut > cbReq)
             {
@@ -734,6 +804,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
                 IOFreeAligned(pvPageBuf, RT_ALIGN_Z(cbReq, PAGE_SIZE));
             else
                 RTMemTmpFree(pHdr);
+            IPRT_DARWIN_RESTORE_EFL_AC();
         }
     }
     else
@@ -744,7 +815,11 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
         if (pUser)
         {
             if (pvPageBuf)
+            {
+                IPRT_DARWIN_SAVE_EFL_AC();
                 IOFreeAligned(pvPageBuf, RT_ALIGN_Z(cbReq, PAGE_SIZE));
+                IPRT_DARWIN_RESTORE_EFL_AC();
+            }
             else
                 RTMemTmpFree(pHdr);
         }
@@ -762,7 +837,7 @@ static int VBoxDrvDarwinIOCtlSlow(PSUPDRVSESSION pSession, u_long iCmd, caddr_t
  * The SUPDRV IDC entry point.
  *
  * @returns VBox status code, see supdrvIDC.
- * @param   iReq        The request code.
+ * @param   uReq        The request code.
  * @param   pReq        The request.
  */
 DECLEXPORT(int) VBOXCALL SUPDrvDarwinIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
@@ -890,17 +965,22 @@ int VBOXCALL supdrvOSEnableVTx(bool fEnable)
 #ifdef VBOX_WITH_HOST_VMX
     int rc;
     if (   version_major >= 10 /* 10 = 10.6.x = Snow Leopard */
+# ifdef VBOX_WITH_RAW_MODE
         && g_pfnVmxSuspend
         && g_pfnVmxResume
-        && g_pVmxUseCount)
+        && g_pVmxUseCount
+# endif
+       )
     {
+        IPRT_DARWIN_SAVE_EFL_AC();
         if (fEnable)
         {
             /*
              * We screwed up on Yosemite and didn't notice that we weren't
-             * calling host_vmxon.  CR4.VMXE may therefor have been disabled
+             * calling host_vmxon.  CR4.VMXE may therefore have been disabled
              * by us.  So, first time around we make sure it's set so we won't
              * crash in the pre-4.3.28/5.0RC1 upgrade scenario.
+             * See @bugref{7907}.
              */
             static bool volatile g_fDoneCleanup = false;
             if (!g_fDoneCleanup)
@@ -917,7 +997,10 @@ int VBOXCALL supdrvOSEnableVTx(bool fEnable)
                             rc = VERR_VMX_NO_VMX;
                     }
                     if (RT_FAILURE(rc))
+                    {
+                        IPRT_DARWIN_RESTORE_EFL_AC();
                         return rc;
+                    }
                 }
                 g_fDoneCleanup = true;
             }
@@ -953,6 +1036,7 @@ int VBOXCALL supdrvOSEnableVTx(bool fEnable)
             rc = VINF_SUCCESS;
             LogRel(("VBoxDrv: host_vmxoff -> vmx_use_count=%d\n", *g_pVmxUseCount));
         }
+        IPRT_DARWIN_RESTORE_EFL_AC();
     }
     else
     {
@@ -983,7 +1067,9 @@ bool VBOXCALL supdrvOSSuspendVTxOnCpu(void)
     if (   g_pVmxUseCount
         && *g_pVmxUseCount > 0)
     {
+        IPRT_DARWIN_SAVE_EFL_AC();
         g_pfnVmxSuspend();
+        IPRT_DARWIN_RESTORE_EFL_AC();
         return true;
     }
     return false;
@@ -1006,7 +1092,11 @@ void VBOXCALL   supdrvOSResumeVTxOnCpu(bool fSuspended)
      */
     if (   fSuspended
         && g_pfnVmxResume)
+    {
+        IPRT_DARWIN_SAVE_EFL_AC();
         g_pfnVmxResume();
+        IPRT_DARWIN_RESTORE_EFL_AC();
+    }
     else
         Assert(!fSuspended);
 #else
@@ -1022,6 +1112,18 @@ bool VBOXCALL supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt)
 }
 
 
+bool VBOXCALL supdrvOSAreCpusOfflinedOnSuspend(void)
+{
+    /** @todo verify this. */
+    return false;
+}
+
+
+bool VBOXCALL supdrvOSAreTscDeltasInSync(void)
+{
+    return false;
+}
+
 void VBOXCALL   supdrvOSLdrNotifyOpened(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
 {
 #if 1
@@ -1076,6 +1178,191 @@ void VBOXCALL   supdrvOSLdrUnload(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
 }
 
 
+#ifdef SUPDRV_WITH_MSR_PROBER
+
+typedef struct SUPDRVDARWINMSRARGS
+{
+    RTUINT64U       uValue;
+    uint32_t        uMsr;
+    int             rc;
+} SUPDRVDARWINMSRARGS, *PSUPDRVDARWINMSRARGS;
+
+/**
+ * On CPU worker for supdrvOSMsrProberRead.
+ *
+ * @param   idCpu           Ignored.
+ * @param   pvUser1         Pointer to a SUPDRVDARWINMSRARGS.
+ * @param   pvUser2         Ignored.
+ */
+static DECLCALLBACK(void) supdrvDarwinMsrProberReadOnCpu(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+{
+    PSUPDRVDARWINMSRARGS pArgs = (PSUPDRVDARWINMSRARGS)pvUser1;
+    if (g_pfnRdMsr64Carefully)
+        pArgs->rc = g_pfnRdMsr64Carefully(pArgs->uMsr, &pArgs->uValue.u);
+    else if (g_pfnRdMsrCarefully)
+        pArgs->rc = g_pfnRdMsrCarefully(pArgs->uMsr, &pArgs->uValue.s.Lo, &pArgs->uValue.s.Hi);
+    else
+        pArgs->rc = 2;
+    NOREF(idCpu); NOREF(pvUser2);
+}
+
+
+int VBOXCALL    supdrvOSMsrProberRead(uint32_t uMsr, RTCPUID idCpu, uint64_t *puValue)
+{
+    if (!g_pfnRdMsr64Carefully && !g_pfnRdMsrCarefully)
+        return VERR_NOT_SUPPORTED;
+
+    SUPDRVDARWINMSRARGS Args;
+    Args.uMsr     = uMsr;
+    Args.uValue.u = 0;
+    Args.rc       = -1;
+
+    if (idCpu == NIL_RTCPUID)
+    {
+        IPRT_DARWIN_SAVE_EFL_AC();
+        supdrvDarwinMsrProberReadOnCpu(idCpu, &Args, NULL);
+        IPRT_DARWIN_RESTORE_EFL_AC();
+    }
+    else
+    {
+        int rc = RTMpOnSpecific(idCpu, supdrvDarwinMsrProberReadOnCpu, &Args, NULL);
+        if (RT_FAILURE(rc))
+            return rc;
+    }
+
+    if (Args.rc)
+        return VERR_ACCESS_DENIED;
+    *puValue = Args.uValue.u;
+    return VINF_SUCCESS;
+}
+
+
+/**
+ * On CPU worker for supdrvOSMsrProberWrite.
+ *
+ * @param   idCpu           Ignored.
+ * @param   pvUser1         Pointer to a SUPDRVDARWINMSRARGS.
+ * @param   pvUser2         Ignored.
+ */
+static DECLCALLBACK(void) supdrvDarwinMsrProberWriteOnCpu(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+{
+    PSUPDRVDARWINMSRARGS pArgs = (PSUPDRVDARWINMSRARGS)pvUser1;
+    if (g_pfnWrMsr64Carefully)
+        pArgs->rc = g_pfnWrMsr64Carefully(pArgs->uMsr, pArgs->uValue.u);
+    else
+        pArgs->rc = 2;
+    NOREF(idCpu); NOREF(pvUser2);
+}
+
+
+int VBOXCALL    supdrvOSMsrProberWrite(uint32_t uMsr, RTCPUID idCpu, uint64_t uValue)
+{
+    if (!g_pfnWrMsr64Carefully)
+        return VERR_NOT_SUPPORTED;
+
+    SUPDRVDARWINMSRARGS Args;
+    Args.uMsr     = uMsr;
+    Args.uValue.u = uValue;
+    Args.rc       = -1;
+
+    if (idCpu == NIL_RTCPUID)
+    {
+        IPRT_DARWIN_SAVE_EFL_AC();
+        supdrvDarwinMsrProberWriteOnCpu(idCpu, &Args, NULL);
+        IPRT_DARWIN_RESTORE_EFL_AC();
+    }
+    else
+    {
+        int rc = RTMpOnSpecific(idCpu, supdrvDarwinMsrProberWriteOnCpu, &Args, NULL);
+        if (RT_FAILURE(rc))
+            return rc;
+    }
+
+    if (Args.rc)
+        return VERR_ACCESS_DENIED;
+    return VINF_SUCCESS;
+}
+
+
+/**
+ * Worker for supdrvOSMsrProberModify.
+ */
+static DECLCALLBACK(void) supdrvDarwinMsrProberModifyOnCpu(RTCPUID idCpu, void *pvUser1, void *pvUser2)
+{
+    PSUPMSRPROBER               pReq    = (PSUPMSRPROBER)pvUser1;
+    register uint32_t           uMsr    = pReq->u.In.uMsr;
+    bool const                  fFaster = pReq->u.In.enmOp == SUPMSRPROBEROP_MODIFY_FASTER;
+    uint64_t                    uBefore;
+    uint64_t                    uWritten;
+    uint64_t                    uAfter;
+    int                         rcBefore, rcWrite, rcAfter, rcRestore;
+    RTCCUINTREG                 fOldFlags;
+
+    /* Initialize result variables. */
+    uBefore = uWritten = uAfter    = 0;
+    rcWrite = rcAfter  = rcRestore = -1;
+
+    /*
+     * Do the job.
+     */
+    fOldFlags = ASMIntDisableFlags();
+    ASMCompilerBarrier(); /* paranoia */
+    if (!fFaster)
+        ASMWriteBackAndInvalidateCaches();
+
+    rcBefore = g_pfnRdMsr64Carefully(uMsr, &uBefore);
+    if (rcBefore >= 0)
+    {
+        register uint64_t uRestore = uBefore;
+        uWritten  = uRestore;
+        uWritten &= pReq->u.In.uArgs.Modify.fAndMask;
+        uWritten |= pReq->u.In.uArgs.Modify.fOrMask;
+
+        rcWrite   = g_pfnWrMsr64Carefully(uMsr, uWritten);
+        rcAfter   = g_pfnRdMsr64Carefully(uMsr, &uAfter);
+        rcRestore = g_pfnWrMsr64Carefully(uMsr, uRestore);
+
+        if (!fFaster)
+        {
+            ASMWriteBackAndInvalidateCaches();
+            ASMReloadCR3();
+            ASMNopPause();
+        }
+    }
+
+    ASMCompilerBarrier(); /* paranoia */
+    ASMSetFlags(fOldFlags);
+
+    /*
+     * Write out the results.
+     */
+    pReq->u.Out.uResults.Modify.uBefore    = uBefore;
+    pReq->u.Out.uResults.Modify.uWritten   = uWritten;
+    pReq->u.Out.uResults.Modify.uAfter     = uAfter;
+    pReq->u.Out.uResults.Modify.fBeforeGp  = rcBefore  != 0;
+    pReq->u.Out.uResults.Modify.fModifyGp  = rcWrite   != 0;
+    pReq->u.Out.uResults.Modify.fAfterGp   = rcAfter   != 0;
+    pReq->u.Out.uResults.Modify.fRestoreGp = rcRestore != 0;
+    RT_ZERO(pReq->u.Out.uResults.Modify.afReserved);
+}
+
+
+int VBOXCALL    supdrvOSMsrProberModify(RTCPUID idCpu, PSUPMSRPROBER pReq)
+{
+    if (!g_pfnWrMsr64Carefully || !g_pfnRdMsr64Carefully)
+        return VERR_NOT_SUPPORTED;
+    if (idCpu == NIL_RTCPUID)
+    {
+        IPRT_DARWIN_SAVE_EFL_AC();
+        supdrvDarwinMsrProberModifyOnCpu(idCpu, pReq, NULL);
+        IPRT_DARWIN_RESTORE_EFL_AC();
+        return VINF_SUCCESS;
+    }
+    return RTMpOnSpecific(idCpu, supdrvDarwinMsrProberModifyOnCpu, pReq, NULL);
+}
+
+#endif /* SUPDRV_WITH_MSR_PROBER */
+
 /**
  * Resume Bluetooth keyboard.
  * If there is no Bluetooth keyboard device connected to the system we just ignore this.
@@ -1135,9 +1422,10 @@ static void supdrvDarwinResumeBuiltinKbd(void)
  */
 int VBOXCALL    supdrvDarwinResumeSuspendedKbds(void)
 {
+    IPRT_DARWIN_SAVE_EFL_AC();
     supdrvDarwinResumeBuiltinKbd();
     supdrvDarwinResumeBluetoothKbd();
-
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return 0;
 }
 
@@ -1167,6 +1455,7 @@ static int VBoxDrvDarwinErr2DarwinErr(int rc)
     return EPERM;
 }
 
+
 /**
  * Check if the CPU has SMAP support.
  */
@@ -1181,11 +1470,17 @@ static bool vboxdrvDarwinCpuHasSMAP(void)
         if (uEBX & X86_CPUID_STEXT_FEATURE_EBX_SMAP)
             return true;
     }
+#ifdef VBOX_WITH_EFLAGS_AC_SET_IN_VBOXDRV
+    return true;
+#else
     return false;
+#endif
 }
 
+
 RTDECL(int) SUPR0Printf(const char *pszFormat, ...)
 {
+    IPRT_DARWIN_SAVE_EFL_AC();
     va_list     va;
     char        szMsg[512];
 
@@ -1195,16 +1490,20 @@ RTDECL(int) SUPR0Printf(const char *pszFormat, ...)
     szMsg[sizeof(szMsg) - 1] = '\0';
 
     printf("%s", szMsg);
+
+    IPRT_DARWIN_RESTORE_EFL_AC();
     return 0;
 }
 
 
-/**
- * Returns configuration flags of the host kernel.
- */
 SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
 {
-    return 0;
+    uint32_t fFlags = 0;
+    if (g_DevCW.d_ioctl == VBoxDrvDarwinIOCtlSMAP)
+        fFlags |= SUPKERNELFEATURES_SMAP;
+    else
+        Assert(!(ASMGetCR4() & X86_CR4_SMAP));
+    return fFlags;
 }
 
 
@@ -1317,6 +1616,15 @@ bool org_virtualbox_SupDrvClient::initWithTask(task_t OwningTask, void *pvSecuri
 
     if (!OwningTask)
         return false;
+
+    VBOX_RETRIEVE_CUR_PROC_NAME(pszProcName);
+
+    if (u32Type != SUP_DARWIN_IOSERVICE_COOKIE)
+    {
+        LogRelMax(10,("org_virtualbox_SupDrvClient::initWithTask: Bad cookie %#x (%s)\n", u32Type, pszProcName));
+        return false;
+    }
+
     if (IOUserClient::initWithTask(OwningTask, pvSecurityId , u32Type))
     {
         m_Task = OwningTask;
@@ -1376,7 +1684,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
                 else
                     rc = VERR_ALREADY_LOADED;
 
-                RTSpinlockReleaseNoInts(g_Spinlock);
+                RTSpinlockRelease(g_Spinlock);
                 if (RT_SUCCESS(rc))
                 {
                     Log(("org_virtualbox_SupDrvClient::start: created session %p for pid %d\n", m_pSession, (int)RTProcSelf()));
@@ -1439,7 +1747,7 @@ bool org_virtualbox_SupDrvClient::start(IOService *pProvider)
             }
         }
     }
-    RTSpinlockReleaseNoInts(g_Spinlock);
+    RTSpinlockRelease(g_Spinlock);
     if (!pSession)
     {
         Log(("SupDrvClient::sessionClose: pSession == NULL, pid=%d; freed already?\n", (int)Process));
