@@ -1033,7 +1033,7 @@ static NTSTATUS vboxUsbRtUrbSendCompletion(PDEVICE_OBJECT pDevObj, IRP *pIrp, vo
     PUSBSUP_URB pUrbInfo = (PUSBSUP_URB)pContext->pOut;
     PVBOXUSBDEV_EXT pDevExt = pContext->pDevExt;
 
-    if (!pUrb || !pMdlBuf || !pUrbInfo | !pDevExt)
+    if (!pUrb || !pMdlBuf || !pUrbInfo || !pDevExt)
     {
         AssertMsgFailed((__FUNCTION__": Invalid args\n"));
         if (pDevExt)
@@ -1342,7 +1342,14 @@ static NTSTATUS vboxUsbRtUrbSend(PVBOXUSBDEV_EXT pDevExt, PIRP pIrp, PUSBSUP_URB
                 iStartFrame = pPipeInfo->NextScheduledFrame;
                 if ((iFrame < iStartFrame) || (iStartFrame > iFrame + 512))
                     iFrame = iStartFrame;
-                pPipeInfo->NextScheduledFrame = iFrame + pUrbInfo->numIsoPkts;
+                /* For full-speed devices, there must be one transfer per frame (Windows USB
+                 * stack requirement), but URBs can contain multiple packets. For high-speed or
+                 * faster transfers, we expect one URB per frame, regardless of the interval.
+                 */
+                if (pDevExt->Rt.devdescr->bcdUSB < 0x300 && !pDevExt->Rt.fIsHighSpeed)
+                    pPipeInfo->NextScheduledFrame = iFrame + pUrbInfo->numIsoPkts;
+                else
+                    pPipeInfo->NextScheduledFrame = iFrame + 1;
                 pUrb->UrbIsochronousTransfer.StartFrame = iFrame;
                 break;
             }
