@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2014 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -20,9 +20,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_MAIN
 #ifdef STANDALONE_TESTCASE
 # define VBOX_WITH_USB
@@ -38,6 +38,7 @@
 #ifdef VBOX_WITH_USB
 # include <IOKit/usb/IOUSBLib.h>
 # include <IOKit/IOCFPlugIn.h>
+# include <IOKit/storage/IOMedia.h>
 #endif
 
 #include <VBox/log.h>
@@ -61,9 +62,9 @@
 #endif
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** An attempt at catching reference leaks. */
 #define MY_CHECK_CREFS(cRefs)   do { AssertMsg(cRefs < 25, ("%ld\n", cRefs)); NOREF(cRefs); } while (0)
 
@@ -75,9 +76,9 @@
 #define VBOXUSBDEVICE_CLASS_NAME "org_virtualbox_VBoxUSBDevice"
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** The IO Master Port. */
 static mach_port_t g_MasterPort = NULL;
 
@@ -274,7 +275,7 @@ static bool darwinDictDupString(CFDictionaryRef DictRef, CFStringRef KeyStrRef,
     char szBuf[512];
     if (darwinDictGetString(DictRef, KeyStrRef, szBuf, sizeof(szBuf)))
     {
-        *ppsz = RTStrDup(RTStrStrip(szBuf));
+        *ppsz = RTStrDup(szBuf);
         if (*ppsz)
             return true;
     }
@@ -373,7 +374,7 @@ static void darwinDumpDictCallback(const void *pvKey, const void *pvValue, void
             double rd;
             CFIndex iCF;
         } u;
-        memset(&u, 0, sizeof(u));
+        RT_ZERO(u);
         CFNumberType NumType = CFNumberGetType((CFNumberRef)pvValue);
         if (CFNumberGetValue((CFNumberRef)pvValue, NumType, &u))
         {
@@ -736,13 +737,28 @@ static io_object_t darwinFindObjectByClass(io_object_t Object, const char *pszCl
  */
 static bool darwinIsMassStorageInterfaceInUse(io_object_t MSDObj, io_name_t pszNameBuf)
 {
-    io_object_t MediaObj = darwinFindObjectByClass(MSDObj, "IOMedia", pszNameBuf);
+    io_object_t MediaObj = darwinFindObjectByClass(MSDObj, kIOMediaClass, pszNameBuf);
     if (MediaObj)
     {
+        CFMutableDictionaryRef pProperties;
+        kern_return_t krc;
+        bool fInUse = true;
+
+        krc = IORegistryEntryCreateCFProperties(MediaObj, &pProperties, kCFAllocatorDefault, kNilOptions);
+        if (krc == KERN_SUCCESS)
+        {
+            CFBooleanRef pBoolValue = (CFBooleanRef)CFDictionaryGetValue(pProperties, CFSTR(kIOMediaOpenKey));
+            if (pBoolValue)
+                fInUse = CFBooleanGetValue(pBoolValue);
+
+            CFRelease(pProperties);
+        }
+
         /* more checks? */
         IOObjectRelease(MediaObj);
-        return true;
+        return fInUse;
     }
+
     return false;
 }
 
@@ -934,8 +950,17 @@ PUSBDEVICE DarwinGetUSBDevices(void)
                 pCur->bcdUSB = 0;                                           /* we've no idea. */
                 pCur->enmState = USBDEVICESTATE_USED_BY_HOST_CAPTURABLE;    /* just a default, we'll try harder in a bit. */
 
-                AssertBreak(darwinDictGetU8(PropsRef,  CFSTR(kUSBDeviceClass),           &pCur->bDeviceClass));
-                /* skip hubs */
+                /* Skip hubs. On 10.11 beta 3, the root hub simulations does not have a USBDeviceClass property, so
+                   simply ignore failures to retrieve it. */
+                if (!darwinDictGetU8(PropsRef,         CFSTR(kUSBDeviceClass),          &pCur->bDeviceClass))
+                {
+#ifdef VBOX_STRICT
+                    char szTmp[80];
+                    Assert(   darwinDictGetString(PropsRef, CFSTR("IOClassNameOverride"), szTmp, sizeof(szTmp))
+                           && strcmp(szTmp, "IOUSBRootHubDevice") == 0);
+#endif
+                    break;
+                }
                 if (pCur->bDeviceClass == 0x09 /* hub, find a define! */)
                     break;
                 AssertBreak(darwinDictGetU8(PropsRef,  CFSTR(kUSBDeviceSubClass),       &pCur->bDeviceSubClass));
@@ -953,11 +978,12 @@ PUSBDEVICE DarwinGetUSBDevices(void)
                 pCur->pszAddress = RTStrDup(szAddress);
                 AssertBreak(pCur->pszAddress);
                 pCur->bBus = u32LocationId >> 24;
-                AssertBreak(darwinDictGetU8(PropsRef,  CFSTR("PortNum"),                &pCur->bPort));
+                darwinDictGetU8(PropsRef, CFSTR("PortNum"), &pCur->bPort); /* Not present in 10.11 beta 3, so ignore failure. (Is set to zero.) */
                 uint8_t bSpeed;
                 AssertBreak(darwinDictGetU8(PropsRef,  CFSTR(kUSBDevicePropertySpeed),  &bSpeed));
-                Assert(bSpeed <= 2);
-                pCur->enmSpeed = bSpeed == 2 ? USBDEVICESPEED_HIGH
+                Assert(bSpeed <= 3);
+                pCur->enmSpeed = bSpeed == 3 ? USBDEVICESPEED_SUPER
+                               : bSpeed == 2 ? USBDEVICESPEED_HIGH
                                : bSpeed == 1 ? USBDEVICESPEED_FULL
                                : bSpeed == 0 ? USBDEVICESPEED_LOW
                                              : USBDEVICESPEED_UNKNOWN;
@@ -1259,7 +1285,7 @@ int DarwinReEnumerateUSBDevice(PCUSBDEVICE pCur)
 
 
 /**
- * Enumerate the DVD drives returning a FIFO of device name strings.
+ * Enumerate the CD, DVD and BlueRay drives returning a FIFO of device name strings.
  *
  * @returns Pointer to the head.
  *          The caller is responsible for calling RTMemFree() on each of the nodes.
@@ -1269,15 +1295,13 @@ PDARWINDVD DarwinGetDVDDrives(void)
     AssertReturn(darwinOpenMasterPort(), NULL);
 
     /*
-     * Create a matching dictionary for searching for DVD services in the IOKit.
+     * Create a matching dictionary for searching for CD, DVD and BlueRay services in the IOKit.
      *
-     * [If I understand this correctly, plain CDROMs doesn't show up as
-     * IODVDServices. Too keep things simple, we will only support DVDs
-     * until somebody complains about it and we get hardware to test it on.
-     * (Unless I'm much mistaken, there aren't any (orignal) intel macs with
-     * plain cdroms.)]
+     * The idea is to find all the devices which are of class IOCDBlockStorageDevice.
+     * CD devices are represented by IOCDBlockStorageDevice class itself, while DVD and BlueRay ones
+     * have it as a parent class.
      */
-    CFMutableDictionaryRef RefMatchingDict = IOServiceMatching("IODVDServices");
+    CFMutableDictionaryRef RefMatchingDict = IOServiceMatching("IOCDBlockStorageDevice");
     AssertReturn(RefMatchingDict, NULL);
 
     /*
@@ -1289,7 +1313,7 @@ PDARWINDVD DarwinGetDVDDrives(void)
     RefMatchingDict = NULL; /* the reference is consumed by IOServiceGetMatchingServices. */
 
     /*
-     * Enumerate the DVD services.
+     * Enumerate the matching services.
      * (This enumeration must be identical to the one performed in DrvHostBase.cpp.)
      */
     PDARWINDVD pHead = NULL;
@@ -1354,7 +1378,7 @@ PDARWINDVD DarwinGetDVDDrives(void)
                             if (*pszVendor && *pszProduct)
                                 RTStrPrintf(szName, sizeof(szName), "%s %s (#%u)", pszVendor, pszProduct, i);
                             else
-                                RTStrPrintf(szName, sizeof(szName), "%s %s (#%u)", *pszVendor ? pszVendor : pszProduct, i);
+                                RTStrPrintf(szName, sizeof(szName), "%s (#%u)", *pszVendor ? pszVendor : pszProduct, i);
                             break;
                         }
                     }
