@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +16,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 
 #include "USBGetDevices.h"
 
@@ -57,9 +57,10 @@
 #include <string.h>
 #include <unistd.h>
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /** Suffix translation. */
 typedef struct USBSUFF
 {
@@ -82,9 +83,10 @@ typedef struct USBDeviceInfo
     VECTOR_PTR(char *) mvecpszInterfaces;
 } USBDeviceInfo;
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /**
  * Suffixes for the endpoint polling interval.
  */
@@ -357,12 +359,14 @@ static int usbReadSpeed(const char *pszValue, USBDEVICESPEED *pSpd, char **ppszN
 {
     pszValue = RTStrStripL(pszValue);
     /* verified with Linux 2.4.0 ... Linux 2.6.25 */
-    if (!strncmp(pszValue, "1.5", 3))
+    if (!strncmp(pszValue, RT_STR_TUPLE("1.5")))
         *pSpd = USBDEVICESPEED_LOW;
-    else if (!strncmp(pszValue, "12 ", 3))
+    else if (!strncmp(pszValue, RT_STR_TUPLE("12 ")))
         *pSpd = USBDEVICESPEED_FULL;
-    else if (!strncmp(pszValue, "480", 3))
+    else if (!strncmp(pszValue, RT_STR_TUPLE("480")))
         *pSpd = USBDEVICESPEED_HIGH;
+    else if (!strncmp(pszValue, RT_STR_TUPLE("5000")))
+        *pSpd = USBDEVICESPEED_SUPER;
     else
         *pSpd = USBDEVICESPEED_UNKNOWN;
     while (pszValue[0] != '\0' && !RT_C_IS_SPACE(pszValue[0]))
@@ -465,7 +469,8 @@ static USBDEVICESTATE usbDeterminState(PCUSBDEVICE pDevice)
 
 
 /** Just a worker for USBProxyServiceLinux::getDevices that avoids some code duplication. */
-static int addDeviceToChain(PUSBDEVICE pDev, PUSBDEVICE *ppFirst, PUSBDEVICE **pppNext, const char *pcszUsbfsRoot, bool testfs, int rc)
+static int addDeviceToChain(PUSBDEVICE pDev, PUSBDEVICE *ppFirst, PUSBDEVICE **pppNext, const char *pcszUsbfsRoot,
+                            bool testfs, int rc)
 {
     /* usbDeterminState requires the address. */
     PUSBDEVICE pDevNew = (PUSBDEVICE)RTMemDup(pDev, sizeof(*pDev));
@@ -593,7 +598,7 @@ static PUSBDEVICE getDevicesFromUsbfs(const char *pcszUsbfsRoot, bool testfs)
                         deviceFreeMembers(&Dev);
 
                     /* Reset device state */
-                    memset(&Dev, 0, sizeof (Dev));
+                    RT_ZERO(Dev);
                     Dev.enmState = USBDEVICESTATE_UNUSED;
                     cHits = 1;
 
@@ -881,10 +886,10 @@ static int addIfDevice(const char *pcszDevicesRoot,
         return VINF_SUCCESS;
     char szDevPath[RTPATH_MAX];
     ssize_t cchDevPath;
-    cchDevPath = RTLinuxFindDevicePath(devnum, RTFS_TYPE_DEV_CHAR,
-                                       szDevPath, sizeof(szDevPath),
-                                       "%s/%.3d/%.3d",
-                                       pcszDevicesRoot, bus, device);
+    cchDevPath = RTLinuxCheckDevicePath(devnum, RTFS_TYPE_DEV_CHAR,
+                                        szDevPath, sizeof(szDevPath),
+                                        "%s/%.3d/%.3d",
+                                        pcszDevicesRoot, bus, device);
     if (cchDevPath < 0)
         return VINF_SUCCESS;
 
@@ -1085,7 +1090,7 @@ static int USBSysfsEnumerateHostDevices(const char *pcszDevicesRoot,
  *   usb<hub number>
  * and usbfs always returns port number zero.
  *
- * @returns VBox status. pu8Port is set on success.
+ * @returns VBox status code. pu8Port is set on success.
  * @param   pszPath     The sysfs path to parse.
  * @param   pu8Port     Where to store the port number.
  */
@@ -1122,7 +1127,7 @@ static int usbGetPortFromSysfsPath(const char *pszPath, uint8_t *pu8Port)
     if (!pchDash && !pchDot)
     {
         /* No -/. so it must be a root hub. Check that it's usb<something>. */
-        if (strncmp(pszLastComp, "usb", sizeof("usb") - 1) != 0)
+        if (strncmp(pszLastComp, RT_STR_TUPLE("usb")) != 0)
         {
             Log(("usbGetPortFromSysfsPath(%s): failed [2]\n", pszPath));
             return VERR_INVALID_PARAMETER;
@@ -1176,6 +1181,7 @@ DECLINLINE(void) usbLogDevice(PUSBDEVICE pDev)
           : pDev->enmSpeed == USBDEVICESPEED_LOW      ? "1.5 MBit/s"
           : pDev->enmSpeed == USBDEVICESPEED_FULL     ? "12 MBit/s"
           : pDev->enmSpeed == USBDEVICESPEED_HIGH     ? "480 MBit/s"
+          : pDev->enmSpeed == USBDEVICESPEED_SUPER    ? "5.0 GBit/s"
           : pDev->enmSpeed == USBDEVICESPEED_VARIABLE ? "variable"
           :                                             "invalid"));
     Log3(("Number of configurations: %d\n", pDev->bNumConfigurations));
@@ -1262,9 +1268,10 @@ static void fillInDeviceFromSysfs(USBDEVICE *Dev, USBDeviceInfo *pInfo)
     if (cchRead <= 0 || (size_t) cchRead == sizeof(szBuf))
         Dev->enmState = USBDEVICESTATE_UNSUPPORTED;
     else
-        Dev->enmSpeed =   !strcmp(szBuf, "1.5") ? USBDEVICESPEED_LOW
-                        : !strcmp(szBuf, "12")  ? USBDEVICESPEED_FULL
-                        : !strcmp(szBuf, "480") ? USBDEVICESPEED_HIGH
+        Dev->enmSpeed =   !strcmp(szBuf, "1.5")  ? USBDEVICESPEED_LOW
+                        : !strcmp(szBuf, "12")   ? USBDEVICESPEED_FULL
+                        : !strcmp(szBuf, "480")  ? USBDEVICESPEED_HIGH
+                        : !strcmp(szBuf, "5000") ? USBDEVICESPEED_SUPER
                         : USBDEVICESPEED_UNKNOWN;
 
     cchRead = RTLinuxSysFsReadStrFile(szBuf, sizeof(szBuf), "%s/version",
@@ -1420,8 +1427,8 @@ static void *testDLSym(void *handle, const char *symbol)
     if (!s_fHaveInotifyLibC)
         return NULL;
     if (s_fHaveInotifyKernel)
-        return (void *)testInotifyInitGood;
-    return (void *)testInotifyInitBad;
+        return (void *)(uintptr_t)testInotifyInitGood;
+    return (void *)(uintptr_t)testInotifyInitBad;
 }
 
 void TestUSBSetInotifyAvailable(bool fHaveInotifyLibC, bool fHaveInotifyKernel)
@@ -1430,7 +1437,7 @@ void TestUSBSetInotifyAvailable(bool fHaveInotifyLibC, bool fHaveInotifyKernel)
     s_fHaveInotifyKernel = fHaveInotifyKernel;
 }
 # define dlsym testDLSym
-# define close(a) do {} while(0)
+# define close(a) do {} while (0)
 #endif
 
 /** Is inotify available and working on this system?  This is a requirement
@@ -1469,6 +1476,8 @@ static PUSBDEVICE testGetUsbfsDevices(const char *pcszUsbfsRoot, bool testfs)
             pNext->pszAddress = RTStrDup(*pcsz);
         if (!pNext || !pNext->pszAddress)
         {
+            if (pNext)
+                RTMemFree(pNext);
             deviceListFree(&pList);
             return NULL;
         }
@@ -1586,7 +1595,7 @@ void TestUSBSetEnv(const char *pcszEnvUsb, const char *pcszEnvUsbRoot)
  * what is available on the host and what if anything the user has specified
  * in the environment.
  * @returns iprt status value
- * @param  pfUsingUsbfsDevices  on success this will be set to true if 
+ * @param  pfUsingUsbfsDevices  on success this will be set to true if
  *                              the prefered access method is USBFS-like and to
  *                              false if it is sysfs/device node-like
  * @param  ppcszDevicesRoot     on success the root of the tree of USBFS-like
