@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -216,14 +216,40 @@ typedef struct RTLDROPS
      * This entrypoint can be omitted if RTLDROPS::pfnGetSymbolEx() is provided and the special BaseAddress feature isn't supported.
      *
      * @returns iprt status code.
+     * @retval  VERR_LDR_FORWARDER forwarder, use pfnQueryForwarderInfo. Buffer size
+     *          in @a pValue.
      * @param   pMod        Pointer to the loader module structure.
      * @param   pvBits      Pointer to bits returned by RTLDROPS::pfnGetBits(), optional.
      * @param   BaseAddress The image base address to use when calculating the symbol value.
+     * @param   iOrdinal    Symbol table ordinal, UINT32_MAX if the symbol name
+     *                      should be used.
      * @param   pszSymbol   The symbol name.
      * @param   pValue      Where to store the symbol value.
      * @remark  Extended loader feature.
      */
-    DECLCALLBACKMEMBER(int, pfnGetSymbolEx)(PRTLDRMODINTERNAL pMod, const void *pvBits, RTUINTPTR BaseAddress, const char *pszSymbol, RTUINTPTR *pValue);
+    DECLCALLBACKMEMBER(int, pfnGetSymbolEx)(PRTLDRMODINTERNAL pMod, const void *pvBits, RTUINTPTR BaseAddress,
+                                            uint32_t iOrdinal, const char *pszSymbol, RTUINTPTR *pValue);
+
+    /**
+     * Query forwarder information on the specified symbol.
+     *
+     * This is an optional entrypoint.
+     *
+     * @returns iprt status code.
+     * @param   pMod        Pointer to the loader module structure.
+     * @param   pvBits      Pointer to bits returned by RTLDROPS::pfnGetBits(),
+     *                      optional.
+     * @param   iOrdinal    Symbol table ordinal of the forwarded symbol to query.
+     *                      UINT32_MAX if the symbol name should be used.
+     * @param   pszSymbol   The symbol name of the forwarded symbol to query.
+     * @param   pInfo       Where to return the forwarder information.
+     * @param   cbInfo      The size of the pInfo buffer. The pfnGetSymbolEx
+     *                      entrypoint returns the required size in @a pValue when
+     *                      the return code is VERR_LDR_FORWARDER.
+     * @remark  Extended loader feature.
+     */
+    DECLCALLBACKMEMBER(int, pfnQueryForwarderInfo)(PRTLDRMODINTERNAL pMod, const void *pvBits, uint32_t iOrdinal,
+                                                   const char *pszSymbol, PRTLDRIMPORTINFO pInfo, size_t cbInfo);
 
     /**
      * Enumerates the debug info contained in the module.
@@ -296,13 +322,85 @@ typedef struct RTLDROPS
      * @returns IPRT status code.
      *
      * @param   pMod            Pointer to the loader module structure.
-     * @param   iSeg            The segment index.
-     * @param   offSeg          The segment offset.
-     * @param   pRva            Where to return the RVA.
+     * @param   Rva             The RVA to convert.
+     * @param   piSeg           Where to return the segment index.
+     * @param   poffSeg         Where to return the segment offset.
      * @remark  This is an optional entry point that can be NULL.
      */
     DECLCALLBACKMEMBER(int, pfnRvaToSegOffset)(PRTLDRMODINTERNAL pMod, RTLDRADDR Rva, uint32_t *piSeg, PRTLDRADDR poffSeg);
 
+    /**
+     * Reads a debug info part (section) from the image.
+     *
+     * This is primarily needed for getting DWARF sections in ELF image with fixups
+     * applied and won't be required by most other loader backends.
+     *
+     * @returns IPRT status code.
+     *
+     * @param   pMod            Pointer to the loader module structure.
+     * @param   pvBuf           The buffer to read into.
+     * @param   iDbgInfo        The debug info ordinal number if the request
+     *                          corresponds exactly to a debug info part from
+     *                          pfnEnumDbgInfo.  Otherwise, pass UINT32_MAX.
+     * @param   off             The offset into the image file.
+     * @param   cb              The number of bytes to read.
+     * @param   pMod            Pointer to the loader module structure.
+     */
+    DECLCALLBACKMEMBER(int, pfnReadDbgInfo)(PRTLDRMODINTERNAL pMod, uint32_t iDbgInfo, RTFOFF off, size_t cb, void *pvBuf);
+
+    /**
+     * Generic method for querying image properties.
+     *
+     * @returns IPRT status code.
+     * @retval  VERR_NOT_SUPPORTED if the property query isn't supported (either all
+     *          or that specific property).
+     * @retval  VERR_NOT_FOUND the property was not found in the module.
+     *
+     * @param   pMod            Pointer to the loader module structure.
+     * @param   enmProp         The property to query (valid).
+     * @param   pvBits          Pointer to the bits returned by
+     *                          RTLDROPS::pfnGetBits(), optional.
+     * @param   pvBuf           Pointer to the input / output buffer. This is valid.
+     *                          Normally only used for returning data, but in some
+     *                          cases it also holds input.
+     * @param   cbBuf           The size of the buffer (valid as per
+     *                          property).
+     * @param   pcbRet          The number of bytes actually returned.  If
+     *                          VERR_BUFFER_OVERFLOW is returned, this is set to the
+     *                          required buffer size.
+     */
+    DECLCALLBACKMEMBER(int, pfnQueryProp)(PRTLDRMODINTERNAL pMod, RTLDRPROP enmProp, void const *pvBits,
+                                          void *pvBuf, size_t cbBuf, size_t *pcbRet);
+
+    /**
+     * Verify the image signature.
+     *
+     * This may permform additional integrity checks on the image structures that
+     * was not done when opening the image.
+     *
+     * @returns IPRT status code.
+     * @retval  VERR_LDRVI_NOT_SIGNED if not signed.
+     *
+     * @param   pMod            Pointer to the loader module structure.
+     * @param   pfnCallback     Callback that does the signature and certificate
+     *                          verficiation.
+     * @param   pvUser          User argument for the callback.
+     * @param   pErrInfo        Pointer to an error info buffer. Optional.
+     */
+    DECLCALLBACKMEMBER(int, pfnVerifySignature)(PRTLDRMODINTERNAL pMod, PFNRTLDRVALIDATESIGNEDDATA pfnCallback, void *pvUser,
+                                                PRTERRINFO pErrInfo);
+
+    /**
+     * Calculate the image hash according the image signing rules.
+     *
+     * @returns IPRT status code.
+     * @param   pMod            The module handle.
+     * @param   enmDigest       Which kind of digest.
+     * @param   pszDigest       Where to store the image digest.
+     * @param   cbDigest        Size of the buffer @a pszDigest points at.
+     */
+    DECLCALLBACKMEMBER(int, pfnHashImage)(PRTLDRMODINTERNAL pMod, RTDIGESTTYPE enmDigest, char *pszDigest, size_t cbDigest);
+
     /** Dummy entry to make sure we've initialized it all. */
     RTUINT uDummy;
 } RTLDROPS;
@@ -310,87 +408,6 @@ typedef RTLDROPS *PRTLDROPS;
 typedef const RTLDROPS *PCRTLDROPS;
 
 
-/** Pointer to a loader reader instance. */
-typedef struct RTLDRREADER *PRTLDRREADER;
-
-/**
- * Loader image reader instance.
- * The reader will have extra data members following this structure.
- */
-typedef struct RTLDRREADER
-{
-    /** The name of the image provider. */
-    const char *pszName;
-
-    /**
-     * Reads bytes at a give place in the raw image.
-     *
-     * @returns iprt status code.
-     * @param   pReader     Pointer to the reader instance.
-     * @param   pvBuf       Where to store the bits.
-     * @param   cb          Number of bytes to read.
-     * @param   off         Where to start reading relative to the start of the raw image.
-     */
-    DECLCALLBACKMEMBER(int, pfnRead)(PRTLDRREADER pReader, void *pvBuf, size_t cb, RTFOFF off);
-
-    /**
-     * Tells end position of last read.
-     *
-     * @returns position relative to start of the raw image.
-     * @param   pReader     Pointer to the reader instance.
-     */
-    DECLCALLBACKMEMBER(RTFOFF, pfnTell)(PRTLDRREADER pReader);
-
-    /**
-     * Gets the size of the raw image bits.
-     *
-     * @returns size of raw image bits in bytes.
-     * @param   pReader     Pointer to the reader instance.
-     */
-    DECLCALLBACKMEMBER(RTFOFF, pfnSize)(PRTLDRREADER pReader);
-
-    /**
-     * Map the bits into memory.
-     *
-     * The mapping will be freed upon calling pfnDestroy() if not pfnUnmap()
-     * is called before that. The mapping is read only.
-     *
-     * @returns iprt status code.
-     * @param   pReader     Pointer to the reader instance.
-     * @param   ppvBits     Where to store the address of the memory mapping on success.
-     *                      The size of the mapping can be obtained by calling pfnSize().
-     */
-    DECLCALLBACKMEMBER(int, pfnMap)(PRTLDRREADER pReader, const void **ppvBits);
-
-    /**
-     * Unmap bits.
-     *
-     * @returns iprt status code.
-     * @param   pReader     Pointer to the reader instance.
-     * @param   pvBits      Memory pointer returned by pfnMap().
-     */
-    DECLCALLBACKMEMBER(int, pfnUnmap)(PRTLDRREADER pReader, const void *pvBits);
-
-    /**
-     * Gets the most appropriate log name.
-     *
-     * @returns Pointer to readonly log name.
-     * @param   pReader     Pointer to the reader instance.
-     */
-    DECLCALLBACKMEMBER(const char *, pfnLogName)(PRTLDRREADER pReader);
-
-    /**
-     * Releases all resources associated with the reader instance.
-     * The instance is invalid after this call returns.
-     *
-     * @returns iprt status code.
-     * @param   pReader     Pointer to the reader instance.
-     */
-    DECLCALLBACKMEMBER(int, pfnDestroy)(PRTLDRREADER pReader);
-
-} RTLDRREADER;
-
-
 /**
  * Loader module core.
  */
@@ -402,6 +419,16 @@ typedef struct RTLDRMODINTERNAL
     RTLDRSTATE              eState;
     /** Loader ops. */
     PCRTLDROPS              pOps;
+    /** Pointer to the reader instance. This is NULL for native image. */
+    PRTLDRREADER            pReader;
+    /** Image format.  */
+    RTLDRFMT                enmFormat;
+    /** Image type.  */
+    RTLDRTYPE               enmType;
+    /** Image endianness.  */
+    RTLDRENDIAN             enmEndian;
+    /** Image target architecture.  */
+    RTLDRARCH               enmArch;
 } RTLDRMODINTERNAL;
 
 
@@ -418,8 +445,6 @@ DECLINLINE(bool) rtldrIsValid(RTLDRMOD hLdrMod)
         && ((PRTLDRMODINTERNAL)hLdrMod)->u32Magic == RTLDRMOD_MAGIC;
 }
 
-int rtldrOpenWithReader(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phMod);
-
 
 /**
  * Native loader module.
@@ -430,7 +455,11 @@ typedef struct RTLDRMODNATIVE
     RTLDRMODINTERNAL    Core;
     /** The native handle. */
     uintptr_t           hNative;
-} RTLDRMODNATIVE, *PRTLDRMODNATIVE;
+    /** The load flags (RTLDRLOAD_FLAGS_XXX). */
+    uint32_t            fFlags;
+} RTLDRMODNATIVE;
+/** Pointer to a native module. */
+typedef RTLDRMODNATIVE *PRTLDRMODNATIVE;
 
 /** @copydoc RTLDROPS::pfnGetSymbol */
 DECLCALLBACK(int) rtldrNativeGetSymbol(PRTLDRMODINTERNAL pMod, const char *pszSymbol, void **ppvValue);
@@ -443,18 +472,31 @@ DECLCALLBACK(int) rtldrNativeClose(PRTLDRMODINTERNAL pMod);
  * @returns iprt status code.
  * @param   pszFilename     The image filename.
  * @param   phHandle        Where to store the module handle on success.
- * @param   fFlags          See RTLDRFLAGS_.
+ * @param   fFlags          RTLDRLOAD_FLAGS_XXX.
  * @param   pErrInfo        Where to return extended error information. Optional.
  */
 int rtldrNativeLoad(const char *pszFilename, uintptr_t *phHandle, uint32_t fFlags, PRTERRINFO pErrInfo);
 
-int rtldrPEOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, RTFOFF offNtHdrs, PRTLDRMOD phLdrMod);
-int rtldrELFOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod);
-int rtldrkLdrOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod);
+/**
+ * Load a system library.
+ *
+ * @returns iprt status code.
+ * @param   pszFilename     The image filename.
+ * @param   pszExt          Extension to add. NULL if none.
+ * @param   fFlags          RTLDRLOAD_FLAGS_XXX.
+ * @param   phLdrMod        Where to return the module handle on success.
+ */
+int rtldrNativeLoadSystem(const char *pszFilename, const char *pszExt, uint32_t fFlags, PRTLDRMOD phLdrMod);
+
+int rtldrPEOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, RTFOFF offNtHdrs, PRTLDRMOD phLdrMod, PRTERRINFO pErrInfo);
+int rtldrELFOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod, PRTERRINFO pErrInfo);
+int rtldrkLdrOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, PRTLDRMOD phLdrMod, PRTERRINFO pErrInfo);
 /*int rtldrLXOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, RTFOFF offLX, PRTLDRMOD phLdrMod);
 int rtldrMachoOpen(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH enmArch, RTFOFF offSomething, PRTLDRMOD phLdrMod);*/
 
 
+DECLHIDDEN(int) rtLdrReadAt(RTLDRMOD hLdrMod, void *pvBuf, uint32_t iDbgInfo, RTFOFF off, size_t cb);
+
 RT_C_DECLS_END
 
 #endif
