@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2009-2012 Oracle Corporation
+ * Copyright (C) 2009-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,15 +16,16 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <VBox/vmm/vmm.h>
 #include "VMMInternal.h"
 #include <VBox/vmm/vm.h>
 #include <VBox/err.h>
 
 #include <iprt/assert.h>
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/string.h>
 
 
@@ -37,8 +38,8 @@
  *          needs to change it into an assertion.
  *
  *
- * @param   pVM             Pointer to the VM.
- * @param   pVCpu           Pointer to the VMCPU of the calling EMT.
+ * @param   pVM             The cross context VM structure.
+ * @param   pVCpu           The cross context virtual CPU structure of the calling EMT.
  * @param   enmOperation    The operation.
  * @param   uArg            The argument to the operation.
  */
@@ -52,6 +53,7 @@ VMMRZDECL(int) VMMRZCallRing3(PVM pVM, PVMCPU pVCpu, VMMCALLRING3 enmOperation,
     if (RT_UNLIKELY(    pVCpu->vmm.s.cCallRing3Disabled != 0
                     &&  enmOperation != VMMCALLRING3_VM_R0_ASSERTION))
     {
+#ifndef IN_RING0
         /*
          * In most cases, it's sufficient to return a status code which
          * will then be propagated up the code usually encountering several
@@ -64,6 +66,7 @@ VMMRZDECL(int) VMMRZCallRing3(PVM pVM, PVMCPU pVCpu, VMMCALLRING3 enmOperation,
          */
         if (enmOperation != VMMCALLRING3_REM_REPLAY_HANDLER_NOTIFICATIONS)
             return VERR_VMM_RING3_CALL_DISABLED;
+#endif
 #ifdef IN_RC
         RTStrPrintf(g_szRTAssertMsg1, sizeof(pVM->vmm.s.szRing0AssertMsg1),
                     "VMMRZCallRing3: enmOperation=%d uArg=%#llx idCpu=%#x\n", enmOperation, uArg, pVCpu->idCpu);
@@ -83,7 +86,14 @@ VMMRZDECL(int) VMMRZCallRing3(PVM pVM, PVMCPU pVCpu, VMMCALLRING3 enmOperation,
 #ifdef IN_RC
     pVM->vmm.s.pfnRCToHost(VINF_VMM_CALL_HOST);
 #else
-    int rc = vmmR0CallRing3LongJmp(&pVCpu->vmm.s.CallRing3JmpBufR0, VINF_VMM_CALL_HOST);
+    int rc;
+    if (pVCpu->vmm.s.pfnCallRing3CallbackR0)
+    {
+        rc = pVCpu->vmm.s.pfnCallRing3CallbackR0(pVCpu, enmOperation, pVCpu->vmm.s.pvCallRing3CallbackUserR0);
+        if (RT_FAILURE(rc))
+            return rc;
+    }
+    rc = vmmR0CallRing3LongJmp(&pVCpu->vmm.s.CallRing3JmpBufR0, VINF_VMM_CALL_HOST);
     if (RT_FAILURE(rc))
         return rc;
 #endif
@@ -99,7 +109,7 @@ VMMRZDECL(int) VMMRZCallRing3(PVM pVM, PVMCPU pVCpu, VMMCALLRING3 enmOperation,
  *          be passed up the stack, or if that isn't possible then VMMRZCallRing3
  *          needs to change it into an assertion.
  *
- * @param   pVM             Pointer to the VM.
+ * @param   pVM             The cross context VM structure.
  * @param   enmOperation    The operation.
  * @param   uArg            The argument to the operation.
  */
@@ -112,14 +122,18 @@ VMMRZDECL(int) VMMRZCallRing3NoCpu(PVM pVM, VMMCALLRING3 enmOperation, uint64_t
 /**
  * Disables all host calls, except certain fatal ones.
  *
- * @param   pVCpu               The CPU struct for the calling EMT.
+ * @param   pVCpu               The cross context virtual CPU structure of the calling EMT.
  * @thread  EMT.
  */
 VMMRZDECL(void) VMMRZCallRing3Disable(PVMCPU pVCpu)
 {
     VMCPU_ASSERT_EMT(pVCpu);
+#if defined(LOG_ENABLED) && defined(IN_RING0)
+    RTCCUINTREG fFlags = ASMIntDisableFlags(); /* preemption consistency. */
+#endif
+
     Assert(pVCpu->vmm.s.cCallRing3Disabled < 16);
-    if (++pVCpu->vmm.s.cCallRing3Disabled == 1)
+    if (ASMAtomicUoIncU32(&pVCpu->vmm.s.cCallRing3Disabled) == 1)
     {
         /** @todo it might make more sense to just disable logging here, then we
          * won't flush away important bits... but that goes both ways really. */
@@ -132,20 +146,28 @@ VMMRZDECL(void) VMMRZCallRing3Disable(PVMCPU pVCpu)
 # endif
 #endif
     }
+
+#if defined(LOG_ENABLED) && defined(IN_RING0)
+    ASMSetFlags(fFlags);
+#endif
 }
 
 
 /**
- * Counters VMMRZCallRing3Disable and re-enables host calls.
+ * Counters VMMRZCallRing3Disable() and re-enables host calls.
  *
- * @param   pVCpu               The CPU struct for the calling EMT.
+ * @param   pVCpu               The cross context virtual CPU structure of the calling EMT.
  * @thread  EMT.
  */
 VMMRZDECL(void) VMMRZCallRing3Enable(PVMCPU pVCpu)
 {
     VMCPU_ASSERT_EMT(pVCpu);
+#if defined(LOG_ENABLED) && defined(IN_RING0)
+    RTCCUINTREG fFlags = ASMIntDisableFlags(); /* preemption consistency. */
+#endif
+
     Assert(pVCpu->vmm.s.cCallRing3Disabled > 0);
-    if (--pVCpu->vmm.s.cCallRing3Disabled == 0)
+    if (ASMAtomicUoDecU32(&pVCpu->vmm.s.cCallRing3Disabled) == 0)
     {
 #ifdef IN_RC
         pVCpu->pVMRC->vmm.s.fRCLoggerFlushingDisabled = false;
@@ -156,6 +178,10 @@ VMMRZDECL(void) VMMRZCallRing3Enable(PVMCPU pVCpu)
 # endif
 #endif
     }
+
+#if defined(LOG_ENABLED) && defined(IN_RING0)
+    ASMSetFlags(fFlags);
+#endif
 }
 
 
@@ -163,7 +189,7 @@ VMMRZDECL(void) VMMRZCallRing3Enable(PVMCPU pVCpu)
  * Checks whether its possible to call host context or not.
  *
  * @returns true if it's safe, false if it isn't.
- * @param   pVCpu               The CPU struct for the calling EMT.
+ * @param   pVCpu               The cross context virtual CPU structure of the calling EMT.
  */
 VMMRZDECL(bool) VMMRZCallRing3IsEnabled(PVMCPU pVCpu)
 {
@@ -172,3 +198,49 @@ VMMRZDECL(bool) VMMRZCallRing3IsEnabled(PVMCPU pVCpu)
     return pVCpu->vmm.s.cCallRing3Disabled == 0;
 }
 
+
+/**
+ * Sets the ring-0 callback before doing the ring-3 call.
+ *
+ * @param   pVCpu         The cross context virtual CPU structure.
+ * @param   pfnCallback   Pointer to the callback.
+ * @param   pvUser        The user argument.
+ *
+ * @return VBox status code.
+ */
+VMMRZDECL(int) VMMRZCallRing3SetNotification(PVMCPU pVCpu, R0PTRTYPE(PFNVMMR0CALLRING3NOTIFICATION) pfnCallback, RTR0PTR pvUser)
+{
+    AssertPtrReturn(pVCpu, VERR_INVALID_POINTER);
+    AssertPtrReturn(pfnCallback, VERR_INVALID_POINTER);
+
+    if (pVCpu->vmm.s.pfnCallRing3CallbackR0)
+        return VERR_ALREADY_EXISTS;
+
+    pVCpu->vmm.s.pfnCallRing3CallbackR0    = pfnCallback;
+    pVCpu->vmm.s.pvCallRing3CallbackUserR0 = pvUser;
+    return VINF_SUCCESS;
+}
+
+
+/**
+ * Removes the ring-0 callback.
+ *
+ * @param   pVCpu   The cross context virtual CPU structure.
+ */
+VMMRZDECL(void) VMMRZCallRing3RemoveNotification(PVMCPU pVCpu)
+{
+    pVCpu->vmm.s.pfnCallRing3CallbackR0 = NULL;
+}
+
+
+/**
+ * Checks whether there is a ring-0 callback notification active.
+ *
+ * @param   pVCpu   The cross context virtual CPU structure.
+ * @returns true if there the notification is active, false otherwise.
+ */
+VMMRZDECL(bool) VMMRZCallRing3IsNotificationSet(PVMCPU pVCpu)
+{
+    return pVCpu->vmm.s.pfnCallRing3CallbackR0 != NULL;
+}
+
