@@ -4,8 +4,8 @@
  * xmlversion.h.in template.
  */
 
-#ifdef WIN32
-#include <win32xmlversion.h>
+#ifdef RT_OS_WINDOWS
+# include <libxml/xmlversion-win.h>
 #else
-#include <libxml/xmlversion-default.h>
+# include <libxml/xmlversion-default.h>
 #endif
